SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[AccountSubAccountVCCBalanceAllocation] (
	@Id VARCHAR(100) = NULL,
	@Step INT = 0,
	@PlatformHash VARCHAR(100) = NULL,
	@InstallationId VARCHAR(50) = NULL,
	@AccountId VARCHAR(50) = NULL,
	@SubAccountId VARCHAR(50) = NULL,
	@VCC VARCHAR(50) = NULL,
	@Balance DECIMAL(18,5) = 0,
	@Target VARCHAR(50) = NULL,
	@Operation VARCHAR(50) = NULL,
	@User VARCHAR(50) = NULL
	)
AS
BEGIN
	-- ESTE SP TIENE LA FINALIDAD DE ACTUALIZAR LOS SALDOS DE LAS CARTERAS DEL VCC
	SET XACT_ABORT, NOCOUNT ON

	-- INICIAMOS UN TRY PARA EL CASO CAIGA EN CATCH HAGAMOS ROLLBACK
    BEGIN TRY

		-- INICIAMOS LA TRANSACCION
		BEGIN TRANSACTION

		-- VERIFICAMOS SI ACCOUNTID TIENE VALOR
		IF @AccountId IS NULL
			-- LANZAMOS MENSAJE AVISANDO QUE FALTA ACCOUNTID
			RAISERROR('DATABABASE_ERROR_MISSING_ACCOUNT_IDENTIFIER_PARAMETER', 16, 16)

		-- VERIFICAMOS QUE TENEMOS LOS PARAMETROS NECESARIOS
		IF @PlatformHash IS NULL OR @InstallationId IS NULL OR @AccountId IS NULL OR @VCC IS NULL
			-- LANZAMOS MENSAJE AVISANDO QUE FALTA ACCOUNTID
			RAISERROR('DATABABASE_ERROR_MISSING_PARAMETERS', 16, 16)

		IF @Operation <> 'INCREASE' AND @Operation <> 'DECREASE'
			-- LANZAMOS MENSAJE AVISANDO QUE HAN SOLICITADO UNA OPERACION DESCONOCIDA
			RAISERROR('DATABABASE_ERROR_UNKNOWN_OPERATION', 16, 16)

		-- DECLARAMOS LA VARIABLE EN LA QUE ESCRIBIREMOS LA DESCRIPCION DEL MOVIMIENTO
		DECLARE @MOVEMENT_DESCRIPTION VARCHAR(250) = NULL

		-- SIEMPRE REGISTRAMOS LOS MOVIMIENTOS INCLUSO SI EL SALDO A AGREGAR ES CERO
		IF @Balance >= 0
		BEGIN
			-- VALIDAMOS SI EXISTE LA CUENTA INGRESADA
			IF NOT EXISTS (
				SELECT AB.[ID] 
					FROM [dbo].[AccountBalance] AB (NOLOCK)
					WHERE AB.[PlatformHash] = @PlatformHash
						AND AB.[InstallationId] = @InstallationId 
						AND AB.[AccountId] = @AccountId
						AND AB.[SubAccountId] IS NULL
			)
				-- LANZAMOS MESAJE AVISANDO QUE NO SE ENCONTRO LA CUENTA
				RAISERROR('DATABABASE_ERROR_NOT_FOUND_ACCOUNT', 16, 16)

			-- EN CASO SUBACCOUNT SEA DIFERENTE DE NULO
			IF @SubAccountId IS NOT NULL 
				IF NOT EXISTS (
					-- BUSCAMOS EL IDENTIFICADOR UNICO DE LA SUBCUENTA EN LA TABLA ACCOUNTBALANCE
					SELECT AB.[ID] 
						FROM [dbo].[AccountBalance] AB (NOLOCK)
						WHERE AB.[PlatformHash] = @PlatformHash
							AND AB.[InstallationId] = @InstallationId 
							AND AB.[AccountId] = @AccountId
							AND AB.[SubAccountId] = @SubAccountId
				)
					-- LANZAMOS MESAJE AVISANDO QUE NO SE ENCONTRO LA CUENTA
					RAISERROR('DATABABASE_ERROR_NOT_FOUND_SUBACCOUNT', 16, 16)

			-- VALIDAMOS SI EXISTE EL VCC CUANDO SUBACCOUNT ES NULO
			IF @SubAccountId IS NULL
			BEGIN
				-- VALIDAMOS SI EXISTE EL VCC
				IF NOT EXISTS (
					SELECT 1 
						FROM [dbo].[AccountVCC] AV (NOLOCK)
						WHERE AV.[InstallationId] = @InstallationId 
							AND AV.[AccountId] = @AccountId
							AND AV.[VCC] = @VCC
				)
					-- LANZAMOS MESAJE AVISANDO QUE NO SE ENCONTRO EL VCC
					RAISERROR('DATABABASE_ERROR_NOT_FOUND_VCC', 16, 16)
			END
			-- VALIDAMOS SI EXISTE EL VCC CUANDO SUBACCOUNT ES DIFERENTE DE NULO
			ELSE 
			BEGIN
				-- VALIDAMOS SI EXISTE EL VCC
				IF NOT EXISTS (
					SELECT 1 
						FROM [dbo].[AccountVCC] AV (NOLOCK)
						WHERE AV.[InstallationId] = @InstallationId 
							AND AV.[AccountId] = @AccountId
							AND AV.[SubAccountId] = @SubAccountId
							AND AV.[VCC] = @VCC
				)
					-- LANZAMOS MESAJE AVISANDO QUE NO SE ENCONTRO EL VCC
					RAISERROR('DATABABASE_ERROR_NOT_FOUND_VCC', 16, 16)
			END

			-- PARA VALIDAR QUE EXISTE LA CARTERA SOLICITADA BASTA CON CONSULTAR CUENTA Y VCC
			IF NOT EXISTS (
				SELECT 1 
					FROM [dbo].[AccountVCCBalance] AB (NOLOCK)
					WHERE AB.[InstallationId] = @InstallationId 
						AND AB.[AccountId] = @AccountId
						AND AB.[VCC] = @VCC
						AND AB.[SystemService] = @Target
			)
				-- LANZAMOS MESAJE AVISANDO QUE NO SE ENCONTRO LA CUENTA
				RAISERROR('DATABABASE_ERROR_NOT_FOUND_TARGET', 16, 16)
				
			-- PASADAS LAS VALIDACIONES BASICAS, APUNTAMOS A HACER LA ACTUALIZACION DE SALDOS

			-- PARA EL CASO QUE ESTEMOS REDUCIENDO EL SALDO, SALE DE TARGET -> FREE
			IF @Operation = 'DECREASE'
			BEGIN
				-- VALIDAMOS QUE EL VCC TIENE SADO SUFICIENTE EL EL SERVICIO SELECCIONADO
				IF 1 = (
					SELECT 1
					FROM [dbo].[AccountVCCBalance] (NOLOCK)
					WHERE [InstallationId] = @InstallationId 
						AND [AccountId] = @AccountId
						AND [VCC] = @VCC
						AND [SystemService] = @Target
						AND [AvailableBalance] >= @Balance
				)
				BEGIN
					-- BLOQUEMOS TEMPORALMENTE EL REGISTRO DE SALDOS DEL SERVICO USANDO LOS MISMO PARAMETROS DE BUSQUEDA Y 
					-- ASEGURANDONOS QUE EL SALDO DISPONIBLE AUN ES MAYOR QUE EL SALDO A SETTEAR
					UPDATE [dbo].[AccountVCCBalance] 
						SET [IsLocked] = 1,
							[LastModifiedByUserId] = @User,
							[LastModifiedDate] = GETUTCDATE(),
							[TimeStamp] = GETUTCDATE() 
						WHERE [InstallationId] = @InstallationId 
							AND [AccountId] = @AccountId
							AND [VCC] = @VCC
							AND [SystemService] = @Target
							AND [AvailableBalance] >= @Balance
	
					IF @@ROWCOUNT = 0
						RAISERROR('DATABABASE_ERROR_INSUFFICIENT_SERVICE_BALANCE', 16, 16)

					-- UNA VEZ BLOQUEADO EL REGISTRO DE LA TABLA ACCOUNTVCCBALANCE, REGISTRAMOS LAS SALIDAS Y ENTRADAS EN 
					-- LA TABLA DE MOVIMIENTOS ACCOUNSUBACCOUNTVCCBALANCEMOVEMENT (MOVIMIENTO DE SERVICIO TARGET -> FREE)
					-- DEFINIMOS LA DESCRIPCION DEL MOVIMIENTO PARA ESTE BLOQUE
					SET @MOVEMENT_DESCRIPTION = CONCAT('VCC ', @VCC, ': DISCOUNT $', @Balance, ' FROM ', @Target, ' WALLET TO FREE WALLET FROM BILLING BALANCE ALLOCATION MODULE') 

					-- INSERTAMOS EN LA TABLA ACCOUNSUBACCOUNTVCCBALANCEMOVEMENT EL REGISTRO PARA LA SALIDA DE LA CARTERA TARGET
					INSERT INTO [dbo].[AccountSubAccountVCCBalanceMovement] 
								([ID],[InstallationId],[AccountId],[SubAccountId],[VCC]
								,[Wallet],[Type],[CurrentStep],[TotalSteps],[Description]
								,[Amount],[PrevAvailableBalance],[PostAvailableBalance],[PrevCountableBalance],[PostCountableBalance])
						SELECT @Id,AVB.[InstallationId],AVB.[AccountId],AVB.[SubAccountId],AVB.[VCC]
								,AVB.[SystemService],'DISCOUNT',(@Step*2)+1,6,@MOVEMENT_DESCRIPTION
								,@Balance,AVB.[AvailableBalance], AVB.[AvailableBalance] - @Balance, AVB.[CountableBalance], AVB.[CountableBalance] - @Balance
							FROM [dbo].[AccountVCCBalance] AVB (NOLOCK)
							WHERE [InstallationId] = @InstallationId 
								AND [AccountId] = @AccountId
								AND [VCC] = @VCC
								AND [SystemService] = @Target
								AND [AvailableBalance] > @Balance

					-- INSERTAMOS EN LA TABLA ACCOUNSUBACCOUNTVCCBALANCEMOVEMENT EL REGISTRO PARA LA ENTRADA A LA CARTERA FREE
					INSERT INTO [dbo].[AccountSubAccountVCCBalanceMovement] 
								([ID],[InstallationId],[AccountId],[SubAccountId],[VCC]
								,[Wallet],[Type],[CurrentStep],[TotalSteps],[Description]
								,[Amount],[PrevAvailableBalance],[PostAvailableBalance],[PrevCountableBalance],[PostCountableBalance])
						SELECT @Id,AVB.[InstallationId],AVB.[AccountId],AVB.[SubAccountId],AVB.[VCC]
								,AVB.[SystemService],'CHARGE',(@Step*2)+2,6,@MOVEMENT_DESCRIPTION
								,@Balance,AVB.[AvailableBalance],AVB.[AvailableBalance] + @Balance, AVB.[CountableBalance], AVB.[CountableBalance] + @Balance
							FROM [dbo].[AccountVCCBalance] AVB (NOLOCK)
							WHERE [InstallationId] = @InstallationId 
								AND [AccountId] = @AccountId
								AND [VCC] = @VCC
								AND [SystemService] = 'FREE'

					-- DESBLOQUEMOS EL REGISTRO DE SALDOS DEL SERVICO USANDO LOS MISMO PARAMETROS DE BUSQUEDA Y ACTUALIZAMOS EL SALDO DISPONIBLE Y CONTABLE
					UPDATE [dbo].[AccountVCCBalance] 
						SET [IsLocked] = 0, 
							[AvailableBalance] = [AvailableBalance] - @Balance,
							[CountableBalance] = [CountableBalance] - @Balance,
							[LastModifiedByUserId] = @User,
							[LastModifiedDate] = GETUTCDATE(),
							[TimeStamp] = GETUTCDATE() 
						WHERE [InstallationId] = @InstallationId 
							AND [AccountId] = @AccountId
							AND [VCC] = @VCC
							AND [SystemService] = @Target
							AND [AvailableBalance] >= @Balance

					IF @@ROWCOUNT = 0
						RAISERROR('DATABABASE_ERROR_INSUFFICIENT_SERVICE_BALANCE', 16, 16)

					-- ACTUALIZAMOS EL REGISTRO DE SALDO DEL FREE
					UPDATE [dbo].[AccountVCCBalance] 
						SET [AvailableBalance] = [AvailableBalance] + @Balance,
							[CountableBalance] = [CountableBalance] + @Balance,
							[LastModifiedByUserId] = @User,
							[LastModifiedDate] = GETUTCDATE(),
							[TimeStamp] = GETUTCDATE() 
						WHERE [InstallationId] = @InstallationId 
							AND [AccountId] = @AccountId
							AND [VCC] = @VCC
							AND [SystemService] = 'FREE'

				END 
				ELSE
					-- LANZAMOS MESAJE AVISANDO QUE NO HAY SUFICIENTE SALDO DISPONIBLE EN EL SERVICIO DEL SISTEMA
					RAISERROR('DATABABASE_ERROR_INSUFFICIENT_SERVICE_AVAILABLE_BALANCE', 16, 16)

			END 

			-- PARA EL CASO QUE ESTEMOS AUMENTANDO EL SALDO, SALE DE  FREE -> TARGET
			ELSE IF @Operation = 'INCREASE' 
			BEGIN
				-- VALIDAMOS QUE EL VCC TIENE SADO SUFICIENTE EN LA CARTERA FREE
				IF 1 = (
					SELECT 1
					FROM [dbo].[AccountVCCBalance] (NOLOCK)
					WHERE [InstallationId] = @InstallationId 
						AND [AccountId] = @AccountId
						AND [VCC] = @VCC
						AND [SystemService] = 'FREE'
						AND [AvailableBalance] >= @Balance
				)
				BEGIN
					-- BLOQUEMOS TEMPORALMENTE EL REGISTRO DE SALDOS DE LA CARTERA FREE USANDO LOS MISMO PARAMETROS DE BUSQUEDA Y 
					-- ASEGURANDONOS QUE EL SALDO DISPONIBLE AUN ES MAYOR QUE EL SALDO A SETTEAR
					UPDATE [dbo].[AccountVCCBalance] 
						SET [IsLocked] = 1,
							[LastModifiedByUserId] = @User,
							[LastModifiedDate] = GETUTCDATE(),
							[TimeStamp] = GETUTCDATE() 
						WHERE [InstallationId] = @InstallationId 
							AND [AccountId] = @AccountId
							AND [VCC] = @VCC
							AND [SystemService] = 'FREE'
							AND [AvailableBalance] >= @Balance
	
					IF @@ROWCOUNT = 0
						RAISERROR('DATABABASE_ERROR_INSUFFICIENT_FREE_BALANCE', 16, 16)

					-- UNA VEZ BLOQUEADO EL REGISTRO DE LA TABLA ACCOUNTVCCBALANCE, REGISTRAMOS LAS SALIDAS Y ENTRADAS EN 
					-- LA TABLA DE MOVIMIENTOS ACCOUNSUBACCOUNTVCCBALANCEMOVEMENT (MOVIMIENTO DE SERVICIO TARGET -> FREE)
					-- DEFINIMOS LA DESCRIPCION DEL MOVIMIENTO PARA ESTE BLOQUE
					SET @MOVEMENT_DESCRIPTION = CONCAT('VCC ', @VCC, ': CHARGE $', @Balance, ' FROM FREE WALLET TO ', @Target, ' WALLET FROM BILLING BALANCE ALLOCATION MODULE') 

					-- INSERTAMOS EN LA TABLA ACCOUNSUBACCOUNTVCCBALANCEMOVEMENT EL REGISTRO PARA LA SALIDA DE LA CARTERA FREE
					INSERT INTO [dbo].[AccountSubAccountVCCBalanceMovement] 
								([ID],[InstallationId],[AccountId],[SubAccountId],[VCC]
								,[Wallet],[Type],[CurrentStep],[TotalSteps],[Description]
								,[Amount],[PrevAvailableBalance],[PostAvailableBalance],[PrevCountableBalance],[PostCountableBalance])
						SELECT @Id,AVB.[InstallationId],AVB.[AccountId],AVB.[SubAccountId],AVB.[VCC]
								,AVB.[SystemService],'CHARGE',(@Step*2)+1,6,@MOVEMENT_DESCRIPTION
								,@Balance,AVB.[AvailableBalance], AVB.[AvailableBalance] - @Balance, AVB.[CountableBalance], AVB.[CountableBalance] - @Balance
							FROM [dbo].[AccountVCCBalance] AVB (NOLOCK)
							WHERE [InstallationId] = @InstallationId 
								AND [AccountId] = @AccountId
								AND [VCC] = @VCC
								AND [SystemService] = 'FREE'
								AND [AvailableBalance] > @Balance

					-- INSERTAMOS EN LA TABLA ACCOUNSUBACCOUNTVCCBALANCEMOVEMENT EL REGISTRO PARA LA ENTRADA A LA CARTERA TARGET
					INSERT INTO [dbo].[AccountSubAccountVCCBalanceMovement] 
								([ID],[InstallationId],[AccountId],[SubAccountId],[VCC]
								,[Wallet],[Type],[CurrentStep],[TotalSteps],[Description]
								,[Amount],[PrevAvailableBalance],[PostAvailableBalance],[PrevCountableBalance],[PostCountableBalance])
						SELECT @Id,AVB.[InstallationId],AVB.[AccountId],AVB.[SubAccountId],AVB.[VCC]
								,AVB.[SystemService],'DISCOUNT',(@Step*2)+2,6,@MOVEMENT_DESCRIPTION
								,@Balance,AVB.[AvailableBalance],AVB.[AvailableBalance] + @Balance, AVB.[CountableBalance], AVB.[CountableBalance] + @Balance
							FROM [dbo].[AccountVCCBalance] AVB (NOLOCK)
							WHERE [InstallationId] = @InstallationId 
								AND [AccountId] = @AccountId
								AND [VCC] = @VCC
								AND [SystemService] = @Target

					-- DESBLOQUEMOS EL REGISTRO DE SALDOS DE LA CARTERA FREE USANDO LOS MISMO PARAMETROS DE BUSQUEDA Y ACTUALIZAMOS EL SALDO DISPONIBLE Y CONTABLE
					UPDATE [dbo].[AccountVCCBalance] 
						SET [IsLocked] = 0, 
							[AvailableBalance] = [AvailableBalance] - @Balance,
							[CountableBalance] = [CountableBalance] - @Balance
						WHERE [InstallationId] = @InstallationId 
							AND [AccountId] = @AccountId
							AND [VCC] = @VCC
							AND [SystemService] = 'FREE'
							AND [AvailableBalance] >= @Balance

					IF @@ROWCOUNT = 0
						RAISERROR('DATABABASE_ERROR_INSUFFICIENT_FREE_BALANCE', 16, 16)

					-- ACTUALIZAMOS EL REGISTRO DE SALDO DEL FREE
					UPDATE [dbo].[AccountVCCBalance] 
						SET [AvailableBalance] = [AvailableBalance] + @Balance,
							[CountableBalance] = [CountableBalance] + @Balance
						WHERE [InstallationId] = @InstallationId 
							AND [AccountId] = @AccountId
							AND [VCC] = @VCC
							AND [SystemService] = @Target

				END 
				ELSE
					-- LANZAMOS MESAJE AVISANDO QUE NO HAY SUFICIENTE SALDO DISPONIBLE EN EL SERVICIO DEL SISTEMA
					RAISERROR('DATABABASE_ERROR_INSUFFICIENT_SERVICE_AVAILABLE_BALANCE', 16, 16)
			END

			-- PARA EL CASO QUE EL SALDO NO AUMENTE NI DISMINUYE
			ELSE IF @Operation = 'NONE' 
			BEGIN
				-- DEFINIMOS LA DESCRIPCION DEL MOVIMIENTO PARA ESTE BLOQUE
				SET @MOVEMENT_DESCRIPTION = CONCAT('VCC ', @VCC, ': NO OPERATION ($', @Balance, ') ON ', @Target, ' WALLET FROM BILLING BALANCE ALLOCATION MODULE') 

			END
			ELSE 
				-- LANZAMOS MESAJE AVISANDO QUE ES UNA OPERACION NO SOPORTADA
				RAISERROR('DATABABASE_ERROR_UNSUPPORTED_OPERATION', 16, 16)
			
		END

		-- RETORNAMOS RESULTADO 1
		SELECT 1 'result'

		-- COMPLETAMOS LA TRANSACCION
		COMMIT TRANSACTION

    END TRY
    BEGIN CATCH
        IF @@TRANCOUNT > 0 ROLLBACK TRANSACTION

        DECLARE @Message nvarchar(2048) = ERROR_MESSAGE()
        DECLARE @Severity integer = ERROR_SEVERITY()
        DECLARE @State integer = ERROR_STATE()

        RAISERROR(@Message, @Severity, @State)
        RETURN -1
    END CATCH

END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO
