SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
-- ACCOUNTSUBACCOUNTVCCBALANCREATE

CREATE  PROCEDURE [dbo].[AccountSubAccountVCCBalanceCreate] (
	@PlatformHash VARCHAR(100) = NULL,
	@InstallationId VARCHAR(50) = NULL,
	@AccountId VARCHAR(50) = NULL,
	@SubAccountId VARCHAR(50) = NULL,
	@VCC VARCHAR(50) = NULL,
	@Balance DECIMAL(18,5) = 0,
	@User VARCHAR(50) = NULL
	)
AS
BEGIN
	SET XACT_ABORT, NOCOUNT ON;

	-- INICIAMOS UN TRY PARA EL CASO CAIGA EN CATCH HAGAMOS ROLLBACK
    BEGIN TRY
        
		-- INICIAMOS LA TRANSACCION
		BEGIN TRANSACTION

		-- VERIFICAMOS SI ACCOUNTID TIENE VALOR
		IF @AccountId IS NULL
			-- LANZAMOS MENSAJE AVISANDO QUE FALTA ACCOUNTID
			RAISERROR('DATABABASE_ERROR_MISSING_ACCOUNT_IDENTIFIER_PARAMETER', 16, 16)
		
		-- DECLARAMOS UNA VARIABLE PARA ALMACENAR EL IDENTIFICADOR UNICO DE LA CUENTA EN LA TABLA ACCOUNTBALANCE
		DECLARE @ACCOUNT_ID VARCHAR(50) = NULL
		DECLARE @SUBACCOUNT_ID VARCHAR(50) = NULL
		DECLARE @MOVEMENT_ID VARCHAR(50) = NEWID()
		DECLARE @MOVEMENT_DESCRIPTION VARCHAR(250) = NULL

		-- CUANDO SUBACCOUNT Y VCC SON NULOS ESTAMOS CREANDO UNA CUENTA
		IF @SubAccountId IS NULL AND @VCC IS NULL 
		BEGIN
			-- INSERTAMOS EL NUEVO REGISTRO EN LA TABLA ACCOUNTBALANCE CON @BALANCE COMO SALDO INICIAL
			INSERT INTO [dbo].[AccountBalance]
						([PlatformHash],[ID],[InstallationId],[AccountId]
						,[TotalBalance],[AvailableBalance],[AssignedBalance],[IsLocked],[CreatedDate]
						,[CreatedByUserId],[TimeStamp])
					VALUES
						(@PlatformHash,NEWID(),@InstallationId,@AccountId
						,@Balance,@Balance,0,0,GETUTCDATE()
						,@User,GETUTCDATE())
			
			-- DEFINIMOS LA DESCRIPCION DEL MOVIMIENTO PARA ESTE BLOQUE
			SET @MOVEMENT_DESCRIPTION = CONCAT('ACCOUNT ', @AccountId, ': FIRST CHARGE $', @Balance, ' TO FREE WALLET(ACCOUNT ', @AccountId, ') FROM BILLING CREATE ACCOUNT MODULE') 

			-- INSERTAMOS EN LA TABLA ACCOUNSUBACCOUNTVCCBALANCEMOVEMENT EL REGISTRO PARA EL PRIMER INGRESO EN LA CARTERA FREE
			INSERT INTO [dbo].[AccountSubAccountVCCBalanceMovement] 
						([ID],[InstallationId],[AccountId],[Wallet],[Type]
						,[CurrentStep],[TotalSteps],[Description],[Amount],[PostAvailableBalance]
						,[PostCountableBalance])
					VALUES
						(@MOVEMENT_ID,@InstallationId,@AccountId,'FREE','CHARGE'
						,1,1,@MOVEMENT_DESCRIPTION,@Balance,@Balance
						,@Balance)
			
		END
		-- CUANDO SUBACCOUNT ES DIFERENTE DE NULO Y VCC ES NULO ESTAMOS CREANDO UNA SUBCUENTA
		ELSE IF @SubAccountId IS NOT NULL AND @VCC IS NULL 
		BEGIN
			-- BUSCAMOS EL IDENTIFICADOR UNICO DE LA CUENTA EN LA TABLA ACCOUNTBALANCE
			SELECT @ACCOUNT_ID = AB.[ID] 
				FROM [dbo].[AccountBalance] AB (NOLOCK)
				WHERE AB.[PlatformHash] = @PlatformHash
					AND AB.[InstallationId] = @InstallationId 
					AND AB.[AccountId] = @AccountId
					AND AB.[SubAccountId] IS NULL

			-- VALIDAMOS SI EXISTE EL IDENTIFICADOR UNICO
			IF @ACCOUNT_ID IS NOT NULL
			BEGIN
				-- VERIFICAMOS QUE LA CUENTA TIENE SALDO DISPONIBLE MAYOR O IGUAL AL BALANCE QUE ESTAMOS SETTEANDO
				IF 1 = (SELECT 1 
						FROM [dbo].[AccountBalance] AB 
						WHERE AB.[ID] = @ACCOUNT_ID
							AND AB.[AvailableBalance] >= @Balance) 
				BEGIN
					-- BLOQUEMOS TEMPORALMENTE EL REGISTRO DE SALDOS DE LA CUENTA USANDO EL IDENTIFICADOR UNICO Y ASEGURANDONOS QUE 
					-- EL SALDO DISPONIBLE AUN ES MAYOR QUE EL SALDO A SETTEAR
					UPDATE [dbo].[AccountBalance] 
						SET [IsLocked] = 1 
						WHERE [ID] = @ACCOUNT_ID
							AND [AvailableBalance] >= @Balance

					IF @@ROWCOUNT = 0
						RAISERROR('DATABABASE_ERROR_INSUFFICIENT_ACCOUNT_BALANCE', 16, 16)

					-- INSERTAMOS EL NUEVO REGISTRO EN LA TABLA ACCOUNTBALANCE CON @BALANCE COMO SALDO INICIAL DE LA SUBCUENTA
					INSERT INTO [dbo].[AccountBalance]
								([PlatformHash],[ID],[InstallationId],[AccountId],[SubAccountId]
								,[TotalBalance],[AvailableBalance],[AssignedBalance],[IsLocked],[CreatedDate]
								,[CreatedByUserId],[TimeStamp])
							VALUES
								(@PlatformHash,NEWID(),@InstallationId,@AccountId,@SubAccountId
								,@Balance,@Balance,0,0,GETUTCDATE()
								,@User,GETUTCDATE())

					-- UNA VEZ BLOQUEADO EL REGISTRO DE LA TABLA ACCOUNTBALANCE, REGISTRAMOS LAS SALIDAS Y ENTRADAS EN 
					-- LA TABLA DE MOVIMIENTOS ACCOUNSUBACCOUNTVCCBALANCEMOVEMENT (MOVIMIENTO DE ACCOUNT FREE -> SUBACCOUNT FREE)

					-- DEFINIMOS LA DESCRIPCION DEL MOVIMIENTO PARA ESTE BLOQUE
					SET @MOVEMENT_DESCRIPTION = CONCAT('SUBACCOUNT ', @SubAccountId, ': FIRST CHARGE $', @Balance, ' TO FREE WALLET(SUBACCOUNT ', @SubAccountId, ') FROM FREE WALLET(ACCOUNT ', @AccountId + ')  FROM BILLING CREATE ACCOUNT MODULE') 

					-- EL PRIMER MOVIMIENTO QUE REGISTRAMOS ES LA SALIDA(DISCOUNT) DE LA CARTERA FREE DE LA CUENTA
					INSERT INTO [dbo].[AccountSubAccountVCCBalanceMovement] 
								([ID],[InstallationId],[AccountId],[Wallet],[Type]
								,[CurrentStep],[TotalSteps],[Description],[Amount],[PrevAvailableBalance]
								,[PostAvailableBalance],[PrevCountableBalance],[PostCountableBalance])
						SELECT @MOVEMENT_ID, @InstallationId, @AccountId, 'FREE','DISCOUNT', 
								1, 3, @MOVEMENT_DESCRIPTION,@Balance,AB.[AvailableBalance]
								,AB.[AvailableBalance] - @Balance,AB.[AvailableBalance],AB.[AvailableBalance] - @Balance
							FROM [dbo].[AccountBalance] AB 
							WHERE [ID] = @ACCOUNT_ID
									
					-- EL SEGUNDO MOVIMIENTO QUE REGISTRAMOS ES LA ENTRADA(CHARGE) A LA CARTERA FREE DE LA SUBCUENTA
					INSERT INTO [dbo].[AccountSubAccountVCCBalanceMovement] 
						([ID],[InstallationId],[AccountId],[SubAccountId],[Wallet]
						,[Type],[CurrentStep],[TotalSteps],[Description],[Amount]
						,[PostAvailableBalance],[PostCountableBalance])
					VALUES
						(@MOVEMENT_ID,@InstallationId,@AccountId,@SubAccountId,'FREE'
						,'CHARGE',2,3,@MOVEMENT_DESCRIPTION,@Balance
						,@Balance,@Balance)

					-- EL TERCER MOVIMIENTO QUE REGISTRAMOS ES LA ENTRADA(CHARGE) A LA CARTERA ASSIGNED DE LA CUENTA
					INSERT INTO [dbo].[AccountSubAccountVCCBalanceMovement] 
								([ID],[InstallationId],[AccountId],[Wallet],[Type]
								,[CurrentStep],[TotalSteps],[Description],[Amount],[PrevAvailableBalance]
								,[PostAvailableBalance],[PrevCountableBalance],[PostCountableBalance])
						SELECT @MOVEMENT_ID, @InstallationId, @AccountId, 'ASSIGNED','CHARGE'
								,3 ,3 ,@MOVEMENT_DESCRIPTION ,@Balance,AB.[AssignedBalance],
								AB.[AssignedBalance] + @Balance,AB.[AssignedBalance],AB.[AssignedBalance] + @Balance
							FROM [dbo].[AccountBalance] AB 
							WHERE [ID] = @ACCOUNT_ID
			
					-- DESBLOQUEMOS EL REGISTRO DE SALDOS DE LA CUENTA USANDO EL IDENTIFICADOR UNICO Y ACTUALIZAMOS EL SALDO DISPONIBLE
					UPDATE [dbo].[AccountBalance] 
						SET [IsLocked] = 0, 
							[AvailableBalance] = [AvailableBalance] - @Balance,
							[AssignedBalance] = [AssignedBalance] + @Balance
						WHERE [ID] = @ACCOUNT_ID
							AND [AvailableBalance] >= @Balance

					IF @@ROWCOUNT = 0
						RAISERROR('DATABABASE_ERROR_INSUFFICIENT_ACCOUNT_BALANCE', 16, 16)

				END
				ELSE 
					-- LANZAMOS MESAJE AVISANDO QUE NO SE ENCONTRO LA CUENTA
					RAISERROR('DATABABASE_ERROR_INSUFFICIENT_ACCOUNT_BALANCE', 16, 16)
			END
			ELSE
			BEGIN
				-- LANZAMOS MESAJE AVISANDO QUE NO SE ENCONTRO LA CUENTA
				RAISERROR('DATABABASE_ERROR_NOT_FOUND_ACCOUNT', 16, 16)
			END
		END 
		-- CUANDO SUBACCOUNT ES NULO Y VCC ES DIFERENTE NULO ESTAMOS CREANDO UN VCC QUE PERTENECE A UNA CUENTA
		ELSE IF @SubAccountId IS NULL AND @VCC IS NOT NULL 
		BEGIN
			-- BUSCAMOS EL IDENTIFICADOR UNICO DE LA CUENTA EN LA TABLA ACCOUNTBALANCE
			SELECT @ACCOUNT_ID = AB.[ID] 
				FROM [dbo].[AccountBalance] AB (NOLOCK)
				WHERE AB.[PlatformHash] = @PlatformHash
					AND AB.[InstallationId] = @InstallationId 
					AND AB.[AccountId] = @AccountId
					AND AB.[SubAccountId] IS NULL

			-- VALIDAMOS SI EXISTE EL IDENTIFICADOR UNICO
			IF @ACCOUNT_ID IS NOT NULL
			BEGIN
				-- VERIFICAMOS QUE LA CUENTA TIENE SALDO DISPONIBLE MAYOR O IGUAL AL BALANCE QUE ESTAMOS SETTEANDO
				IF 1 = (SELECT 1 
						FROM [dbo].[AccountBalance] AB (NOLOCK)
						WHERE AB.[ID] = @ACCOUNT_ID
							AND AB.[AvailableBalance] >= @Balance) 
				BEGIN	
					
					-- BLOQUEMOS TEMPORALMENTE EL REGISTRO DE SALDOS DE LA CUENTA USANDO EL IDENTIFICADOR UNICO Y ASEGURANDONOS QUE 
					-- EL SALDO DISPONIBLE AUN ES MAYOR QUE EL SALDO A SETTEAR
					UPDATE [dbo].[AccountBalance] 
						SET [IsLocked] = 1,
							[LastModifiedByUserId] = @User,
							[LastModifiedDate] = GETUTCDATE(),
							[LastModifiedByRemoteUser] = 1
						WHERE [ID] = @ACCOUNT_ID
							AND [AvailableBalance] >= @Balance

					IF @@ROWCOUNT = 0
						RAISERROR('DATABABASE_ERROR_INSUFFICIENT_ACCOUNT_BALANCE', 16, 16)

					-- UNA VEZ BLOQUEADO EL REGISTRO DE LA TABLA ACCOUNTBALANCE, REGISTRAMOS LAS SALIDAS Y ENTRADAS EN LA TABLA DE 
					-- MOVIMIENTOS ACCOUNSUBACCOUNTVCCBALANCEMOVEMENT (MOVIMIENTO DE ACCOUNT FREE -> VCC REE)
					
					-- DEFINIMOS LA DESCRIPCION DEL MOVIMIENTO PARA ESTE BLOQUE
					SET @MOVEMENT_DESCRIPTION = CONCAT('VCC ', @VCC, ': FIRST CHARGE $', @Balance, ' TO FREE WALLET(VCC ', @VCC, ') FROM FREE WALLET(ACCOUNT ', @AccountId + ') FROM BILLING CREATE ACCOUNT MODULE') 

					-- EL PRIMER MOVIMIENTO QUE REGISTRAMOS ES LA SALIDA DE LA CUENTA
					INSERT INTO [dbo].[AccountSubAccountVCCBalanceMovement] 
								([ID],[InstallationId],[AccountId],[Wallet],[Type]
								,[CurrentStep],[TotalSteps],[Description],[Amount],[PrevAvailableBalance]
								,[PostAvailableBalance],[PrevCountableBalance],[PostCountableBalance])
						SELECT @MOVEMENT_ID, @InstallationId, @AccountId, 'FREE','DISCOUNT'
								,1 ,3 ,@MOVEMENT_DESCRIPTION ,@Balance,AB.[AvailableBalance]
								,AB.[AvailableBalance] - @Balance,AB.[AvailableBalance],AB.[AvailableBalance] - @Balance
							FROM [dbo].[AccountBalance] AB (NOLOCK)
							WHERE [ID] = @ACCOUNT_ID
									
					-- EL SEGUNDO MOVIMIENTO QUE REGISTRAMOS ES LA ENTRADA AL VCC
					INSERT INTO [dbo].[AccountSubAccountVCCBalanceMovement] 
						([ID],[InstallationId],[AccountId],[VCC],[Wallet]
						,[Type],[CurrentStep],[TotalSteps],[Description],[Amount]
						,[PostAvailableBalance],[PostCountableBalance])
					VALUES
						(@MOVEMENT_ID,@InstallationId,@AccountId,@VCC,'FREE'
						,'CHARGE',2,3,@MOVEMENT_DESCRIPTION ,@Balance
						,@Balance,@Balance)

					-- EL TERCER MOVIMIENTO QUE REGISTRAMOS ES LA ENTRADA(CHARGE) A LA CARTERA ASSIGNED DE LA CUENTA
					INSERT INTO [dbo].[AccountSubAccountVCCBalanceMovement] 
								([ID],[InstallationId],[AccountId],[Wallet],[Type]
								,[CurrentStep],[TotalSteps],[Description],[Amount],[PrevAvailableBalance]
								,[PostAvailableBalance],[PrevCountableBalance],[PostCountableBalance])
						SELECT @MOVEMENT_ID, @InstallationId, @AccountId, 'ASSIGNED','CHARGE'
								,3,3,@MOVEMENT_DESCRIPTION,@Balance,AB.[AssignedBalance]
								,AB.[AssignedBalance] + @Balance,AB.[AssignedBalance],AB.[AssignedBalance] + @Balance
							FROM [dbo].[AccountBalance] AB 
							WHERE [ID] = @ACCOUNT_ID
			
					-- DESCBLOQUEMOS EL REGISTRO DE SALDOS DE LA CUENTA USANDO EL IDENTIFICADOR UNICO Y ACTUALIZAMOS EL SALDO DISPONIBLE
					UPDATE [dbo].[AccountBalance] 
						SET [IsLocked] = 0, 
							[AvailableBalance] = [AvailableBalance] - @Balance,
							[AssignedBalance] = [AssignedBalance] + @Balance,
							[LastModifiedByUserId] = @User,
							[LastModifiedDate] = GETUTCDATE(),
							[LastModifiedByRemoteUser] = 1
						WHERE [ID] = @ACCOUNT_ID
							AND [AvailableBalance] >= @Balance

					IF @@ROWCOUNT = 0
						RAISERROR('DATABABASE_ERROR_INSUFFICIENT_ACCOUNT_BALANCE', 16, 16)

					-- ACTUALIZAMOS EL REGISTRO DEL WALLET FREE DEL VCC EN LA TABLA ACCOUNTVCCBALANCE
					UPDATE [dbo].[AccountVCCBalance]
						SET [AvailableBalance] = [AvailableBalance] + @Balance,
							[CountableBalance] = [CountableBalance] + @Balance,
							[LastModifiedByUserId] = @User,
							[LastModifiedDate] = GETUTCDATE()
						WHERE [InstallationId] = @InstallationId
							AND [AccountId] = @AccountId
							AND [VCC] = @VCC
							AND [SystemService] = 'FREE'

					IF @@ROWCOUNT = 0
						RAISERROR('DATABABASE_ERROR_ACCOUNTVCC_BALANCE_UPDATE_FAILED', 16, 16)
					
				END
				ELSE 
					-- LANZAMOS MESAJE AVISANDO QUE NO SE ENCONTRO LA CUENTA
					RAISERROR('DATABABASE_ERROR_INSUFFICIENT_ACCOUNT_BALANCE', 16, 16)

			END
			ELSE
			BEGIN
				-- LANZAMOS MESAJE AVISANDO QUE NO SE ENCONTRO LA CUENTA
				RAISERROR('DATABABASE_ERROR_NOT_FOUND_ACCOUNT', 16, 16)
			END
		END
		-- CUANDO SUBACCOUNT ES DIFERENTE DE NULO Y VCC ES DIFERENTE NULO ESTAMOS CREANDO UN VCC QUE PERTENECE A UNA SUBCUENTA
		ELSE IF @SubAccountId IS NOT NULL AND @VCC IS NOT NULL 
		BEGIN
			-- BUSCAMOS EL IDENTIFICADOR UNICO DE LA CUENTA EN LA TABLA ACCOUNTBALANCE
			SELECT @ACCOUNT_ID = AB.[ID] 
				FROM [dbo].[AccountBalance] AB (NOLOCK)
				WHERE AB.[PlatformHash] = @PlatformHash
					AND AB.[InstallationId] = @InstallationId 
					AND AB.[AccountId] = @AccountId
					AND AB.[SubAccountId] IS NULL

			-- VALIDAMOS SI EXISTE EL IDENTIFICADOR UNICO
			IF @ACCOUNT_ID IS NOT NULL
			BEGIN
				-- BUSCAMOS EL IDENTIFICADOR UNICO DE LA CUENTA EN LA TABLA ACCOUNTBALANCE
				SELECT @SUBACCOUNT_ID = AB.[ID] 
					FROM [dbo].[AccountBalance] AB (NOLOCK)
					WHERE AB.[PlatformHash] = @PlatformHash
						AND AB.[InstallationId] = @InstallationId 
						AND AB.[AccountId] = @AccountId
						AND AB.[SubAccountId] = @SubAccountId

				-- VALIDAMOS SI EXISTE EL IDENTIFICADOR UNICO
				IF @SUBACCOUNT_ID IS NOT NULL
				BEGIN
					-- VERIFICAMOS QUE LA CUENTA TIENE SALDO DISPONIBLE MAYOR O IGUAL AL BALANCE QUE ESTAMOS SETTEANDO
					IF 1 = (SELECT 1 
							FROM [dbo].[AccountBalance] AB 
							WHERE AB.[ID] = @SUBACCOUNT_ID
								AND AB.[AvailableBalance] >= @Balance) 
					BEGIN
						-- BLOQUEMOS TEMPORALMENTE EL REGISTRO DE SALDOS DE LA SUBCUENTA USANDO EL IDENTIFICADOR UNICO Y ASEGURANDONOS QUE 
						-- EL SALDO DISPONIBLE AUN ES MAYOR QUE EL SALDO A SETTEAR
						UPDATE [dbo].[AccountBalance] 
							SET [IsLocked] = 1 
							WHERE [ID] = @SUBACCOUNT_ID
								AND [AvailableBalance] >= @Balance
	
						IF @@ROWCOUNT = 0
							RAISERROR('DATABABASE_ERROR_INSUFFICIENT_SUBACCOUNT_BALANCE', 16, 16)

						-- UNA VEZ BLOQUEADO EL REGISTRO DE LA TABLA ACCOUNTBALANCE, REGISTRAMOS LAS SALIDAS Y ENTRADAS EN 
						-- LA TABLA DE MOVIMIENTOS ACCOUNSUBACCOUNTVCCBALANCEMOVEMENT (MOVIMIENTO DE SUBACCOUNT FREE -> VCC FREE)

						-- DEFINIMOS LA DESCRIPCION DEL MOVIMIENTO PARA ESTE BLOQUE
						SET @MOVEMENT_DESCRIPTION = CONCAT('VCC ', @VCC, ': FIRST CHARGE $', @Balance, ' TO FREE WALLET(VCC ', @VCC, ') FROM FREE WALLET(SUBACCOUNT ', @SubAccountId, ') FROM BILLING CREATE ACCOUNT MODULE') 

						-- EL PRIMER MOVIMIENTO QUE REGISTRAMOS ES LA SALIDA(DISCOUNT) DE LA CARTERA FREE DE LA SUBCUENTA
						INSERT INTO [dbo].[AccountSubAccountVCCBalanceMovement] 
									([ID],[InstallationId],[AccountId],[SubAccountId],[Wallet]
									,[Type],[CurrentStep],[TotalSteps],[Description],[Amount]
									,[PrevAvailableBalance],[PostAvailableBalance],[PrevCountableBalance],[PostCountableBalance])
							SELECT @MOVEMENT_ID, @InstallationId, @AccountId, @SubAccountId, 'FREE'
									,'DISCOUNT',1,3,@MOVEMENT_DESCRIPTION,@Balance
									,AB.[AvailableBalance],AB.[AvailableBalance] - @Balance,AB.[AvailableBalance],AB.[AvailableBalance] - @Balance
								FROM [dbo].[AccountBalance] AB 
								WHERE [ID] = @SUBACCOUNT_ID

						-- EL SEGUNDO MOVIMIENTO QUE REGISTRAMOS ES LA ENTRADA(CHARGE) A LA CARTERA FREE DEL VCC
						INSERT INTO [dbo].[AccountSubAccountVCCBalanceMovement] 
							([ID],[InstallationId],[AccountId],[SubAccountId],[VCC]
							,[Wallet],[Type],[CurrentStep],[TotalSteps],[Description]
							,[Amount],[PostAvailableBalance],[PostCountableBalance])
						VALUES
							(@MOVEMENT_ID,@InstallationId,@AccountId,@SubAccountId,@VCC
							,'FREE','CHARGE',2,3,@MOVEMENT_DESCRIPTION
							,@Balance,@Balance,@Balance)

						-- EL TERCER MOVIMIENTO QUE REGISTRAMOS ES LA ENTRADA(CHARGE) A LA CARTERA ASSIGNED DE LA SUBCUENTA
						INSERT INTO [dbo].[AccountSubAccountVCCBalanceMovement] 
									([ID],[InstallationId],[AccountId],[SubAccountId],[Wallet]
									,[Type],[CurrentStep],[TotalSteps],[Description],[Amount]
									,[PrevAvailableBalance],[PostAvailableBalance],[PrevCountableBalance],[PostCountableBalance])
							SELECT @MOVEMENT_ID, @InstallationId, @AccountId,[SubAccountId],'ASSIGNED'
									,'CHARGE', 3, 3, @MOVEMENT_DESCRIPTION, @Balance
									,AB.[AssignedBalance], AB.[AssignedBalance] + @Balance,AB.[AssignedBalance], AB.[AssignedBalance] + @Balance
								FROM [dbo].[AccountBalance] AB 
								WHERE [ID] = @SUBACCOUNT_ID
			
						-- DESBLOQUEMOS EL REGISTRO DE SALDOS DE LA SUBCUENTA USANDO EL IDENTIFICADOR UNICO Y ACTUALIZAMOS EL SALDO DISPONIBLE
						UPDATE [dbo].[AccountBalance] 
							SET [IsLocked] = 0, 
								[AvailableBalance] = [AvailableBalance] - @Balance,
								[AssignedBalance] = [AssignedBalance] + @Balance
							WHERE [ID] = @SUBACCOUNT_ID
								AND [AvailableBalance] >= @Balance

						IF @@ROWCOUNT = 0
							RAISERROR('DATABABASE_ERROR_INSUFFICIENT_SUBACCOUNT_BALANCE', 16, 16)

						-- ACTUALIZAMOS EL REGISTRO DEL WALLET FREE DEL VCC EN LA TABLA ACCOUNTVCCBALANCE
						UPDATE [dbo].[AccountVCCBalance]
							SET [AvailableBalance] = @Balance,
								[CountableBalance] = @Balance,
								[LastModifiedByUserId] = @User,
								[LastModifiedDate] = GETUTCDATE()
							WHERE [InstallationId] = @InstallationId
								AND [AccountId] = @AccountId
								AND [SubAccountId] = @SubAccountId
								AND [VCC] = @VCC
								AND [SystemService] = 'FREE'

						IF @@ROWCOUNT = 0
							RAISERROR('DATABABASE_ERROR_ACCOUNTVCC_BALANCE_UPDATE_FAILED', 16, 16)
					END 
					ELSE 
						-- LANZAMOS MESAJE AVISANDO QUE NO HAY SUFICIENTE SALDO EN LA SUBCUENTA
						RAISERROR('DATABABASE_ERROR_INSUFFICIENT_SUBACCOUNT_BALANCE', 16, 16)
				END
				ELSE 
					-- LANZAMOS MESAJE AVISANDO QUE NO SE ENCONTRO LA SUBCUENTA
					RAISERROR('DATABABASE_ERROR_NOT_FOUND_SUBACCOUNT', 16, 16)
			END
			ELSE 
				-- LANZAMOS MESAJE AVISANDO QUE NO SE ENCONTRO LA CUENTA
				RAISERROR('DATABABASE_ERROR_NOT_FOUND_ACCOUNT', 16, 16)
		END

		-- COMPLETAMOS LA TRANSACCION
		COMMIT TRANSACTION;

    END TRY
    BEGIN CATCH
        IF @@TRANCOUNT > 0 ROLLBACK TRANSACTION;

        DECLARE @Message nvarchar(2048) = ERROR_MESSAGE()
        DECLARE @Severity integer = ERROR_SEVERITY()
        DECLARE @State integer = ERROR_STATE()

        RAISERROR(@Message, @Severity, @State)
        RETURN -1
    END CATCH

END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO
