SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[AccountSubAccountVCCBatchSummaryClose](
	@BatchId VARCHAR(50) = NULL, 
	@ClosedDate DATETIME = NULL
)
AS
BEGIN
	-- ESTE SP TIENE LA FINALIDAD DE ACTUALIZAR LOS SALDOS DE LAS CARTERAS DEL VCC
	SET XACT_ABORT, NOCOUNT ON

	-- INICIAMOS UN TRY PARA EL CASO CAIGA EN CATCH HAGAMOS ROLLBACK
    BEGIN TRY

		-- INICIAMOS LA TRANSACCION
		BEGIN TRANSACTION

		-- VERIFICAMOS QUE TENEMOS LOS PARAMETROS NECESARIOS
		IF @BatchId IS NULL 
			-- LANZAMOS MENSAJE AVISANDO QUE FALTA ACCOUNTID
			RAISERROR('DATABABASE_ERROR_MISSING_PARAMETERS', 16, 16)

		-- DECLARAMOS UNA VARIABLE PARA ALMACENAR LOS SYSTEMSERVICE RELACIONADOS AL BATCH
		DECLARE MY_CURSOR CURSOR 
		LOCAL STATIC READ_ONLY FORWARD_ONLY
			FOR 
			-- CONSULTAMOS LOS SYSTEMSERVICE DEL BATCH PARA LA ACTUALIZACION DE SALDOS
			SELECT ASAVST.[InstallationId],
					ASAVST.[VCC], 
					ASAVST.[SystemService], 
					ASAVST.[Rate]
				FROM [dbo].[AccountSubAccountVCCSummaryTransaction] ASAVST 
				WHERE ASAVST.[BatchID] = @BatchId

		-- DECLARARAMOS UNA VARIABLE PARA INDICAR EL BACTH ACTUAL QUE ESTAMOS RECORRIENDO
		DECLARE @CURRENT_INSTALLATION_ID VARCHAR(50) = ''
		DECLARE @CURRENT_VCC VARCHAR(50) = ''
		DECLARE @CURRENT_SYSTEM_SERVICE VARCHAR(50) = ''
		DECLARE @CURRENT_RATE DECIMAL(18, 5) = 0

		-- DECLARAMOS VARIABLES DE AYUDA QUE CAMBIARAN EN EL BUCLE
		DECLARE @MOVEMENT_DESCRIPTION VARCHAR(250) = NULL
		DECLARE @BATCH_TOTAL_STEPS INT = 0
		DECLARE @BATCH_CURRENT_STEP INT = 0
		DECLARE @INITIAL_CALCULATED_AMOUNT DECIMAL(18,5) = 0
		DECLARE @FINAL_CALCULATED_AMOUNT DECIMAL(18,5) = 0

		
		-- ABRIMOS Y RECORREMOS EL ARRAY DE CAMPOS A ACTUALIZAR
		OPEN MY_CURSOR
		FETCH NEXT FROM MY_CURSOR INTO @CURRENT_INSTALLATION_ID, @CURRENT_VCC, @CURRENT_SYSTEM_SERVICE, @CURRENT_RATE
		WHILE @@FETCH_STATUS = 0
		BEGIN 
			-- OBTENEMOS LOS DATOS GENERALES DEL BATCH EN LA TABLA DE MOVIMIENTOS
			SELECT @BATCH_CURRENT_STEP = MAX(ASAVBM.[CurrentStep]),
					@BATCH_TOTAL_STEPS = ASAVBM.[TotalSteps]
				FROM [dbo].[AccountSubAccountVCCBalanceMovement] ASAVBM (NOLOCK)
				WHERE ASAVBM.[BatchID] = @BatchId
				GROUP BY ASAVBM.[InstallationId], ASAVBM.[VCC], ASAVBM.[TotalSteps]

			-- OBTENEMOS LOS DATOS ESPECIFICOS DEL BATCH-SYSTEMSERVICE EN LA TABLA DE MOVIMIENTOS
			SELECT @INITIAL_CALCULATED_AMOUNT = ASAVBM.[Amount]
				FROM [dbo].[AccountSubAccountVCCBalanceMovement] ASAVBM (NOLOCK)
				WHERE ASAVBM.[BatchID] = @BatchId
					AND ASAVBM.[InstallationId] = @CURRENT_INSTALLATION_ID
					AND ASAVBM.[Wallet] = @CURRENT_SYSTEM_SERVICE

			-- PARA REGISTRAR LOS MOVIMIENTOS AUMENTAMOS EL CURRENT_STEP PARA REGISTRAR EL SIGUIENTE MOVIMIENTO
			SET @BATCH_CURRENT_STEP = @BATCH_CURRENT_STEP + 1

			-- DEFINIMOS LA DESCRIPCION DEL MOVIMIENTO DEBITAJE
			SET @MOVEMENT_DESCRIPTION = CONCAT('VCC ', @CURRENT_VCC, ': DEBT $', @CURRENT_RATE, ' FROM ', @CURRENT_SYSTEM_SERVICE, ' WALLET ON CLOSING BATCH')

			-- INSERTAMOS EN LA TABLA ACCOUNSUBACCOUNTVCCBALANCEMOVEMENT EL REGISTRO PARA EL COBRO A LA CARTERA RECORRIDA
			INSERT INTO [dbo].[AccountSubAccountVCCBalanceMovement] 
						([ID] ,[InstallationId] ,[AccountId] ,[SubAccountId] ,[VCC]
						,[Wallet] ,[Type] ,[CurrentStep] ,[TotalSteps] ,[Description]
						,[BatchId] ,[Amount]
						,[PrevAvailableBalance] ,[PostAvailableBalance] 
						,[PrevCountableBalance] ,[PostCountableBalance] 
						,[TimeStamp])
				SELECT @BatchId ,AVB.[InstallationId] ,AVB.[AccountId] ,AVB.[SubAccountId] ,AVB.[VCC]
						,AVB.[SystemService] ,'DEBT' ,@BATCH_CURRENT_STEP ,@BATCH_TOTAL_STEPS ,@MOVEMENT_DESCRIPTION
						,@BatchId ,@CURRENT_RATE
						,AVB.[AvailableBalance] ,AVB.[AvailableBalance]
						,AVB.[CountableBalance] ,AVB.[CountableBalance] - @CURRENT_RATE 
						,GETUTCDATE()
					FROM [dbo].[AccountVCCBalance] AVB (NOLOCK)
					WHERE [InstallationId] = @CURRENT_INSTALLATION_ID 
						AND [VCC] = @CURRENT_VCC
						AND [SystemService] = @CURRENT_SYSTEM_SERVICE
						--AND [CountableBalance] > @FINAL_CALCULATED_AMOUNT

			-- DESBLOQUEMOS EL REGISTRO DE SALDOS DEL SERVICO USANDO LOS MISMO PARAMETROS DE BUSQUEDA Y ACTUALIZAMOS EL SALDO DISPONIBLE Y CONTABLE
			UPDATE [dbo].[AccountVCCBalance] 
				SET [IsLocked] = 0, 
					[AvailableBalance] = [AvailableBalance] + (@INITIAL_CALCULATED_AMOUNT - @CURRENT_RATE),
					[RetainedBalance] = [RetainedBalance] - @INITIAL_CALCULATED_AMOUNT,
					[CountableBalance] = [CountableBalance] - @CURRENT_RATE,
					--[LastModifiedByUserId] = @UserApi,
					[LastModifiedDate] = GETUTCDATE(),
					[TimeStamp] = GETUTCDATE() 
				WHERE [InstallationId] = @CURRENT_INSTALLATION_ID 
					AND [VCC] = @CURRENT_VCC
					AND [SystemService] = @CURRENT_SYSTEM_SERVICE

			-- AUMENTAMOS EL CURRENT_STEP PARA REGISTRAR EL SIGUIENTE MOVIMIENTO
			SET @BATCH_CURRENT_STEP = @BATCH_CURRENT_STEP + 1

			-- DEFINIMOS LA DESCRIPCION DEL MOVIMIENTO LIBERAR
			SET @MOVEMENT_DESCRIPTION = CONCAT('VCC ', @CURRENT_VCC, ': RELEASE $', (@INITIAL_CALCULATED_AMOUNT - @CURRENT_RATE), ' FROM ', @CURRENT_SYSTEM_SERVICE, ' WALLET ON CLOSING BATCH')
			
			-- INSERTAMOS EN LA TABLA ACCOUNSUBACCOUNTVCCBALANCEMOVEMENT EL REGISTRO PARA LA LIBERACION DE SALDO RETENIDO A LA CARTERA RECORRIDA
			INSERT INTO [dbo].[AccountSubAccountVCCBalanceMovement] 
						([ID] ,[InstallationId] ,[AccountId] ,[SubAccountId] ,[VCC]
						,[Wallet] ,[Type] ,[CurrentStep] ,[TotalSteps] ,[Description]
						,[BatchId] ,[Amount]
						,[PrevAvailableBalance] ,[PostAvailableBalance] 
						,[PrevCountableBalance] ,[PostCountableBalance] 
						,[TimeStamp])
				SELECT @BatchId ,AVB.[InstallationId] ,AVB.[AccountId] ,AVB.[SubAccountId] ,AVB.[VCC]
						,AVB.[SystemService] ,'RELEASE' ,@BATCH_CURRENT_STEP ,@BATCH_TOTAL_STEPS ,@MOVEMENT_DESCRIPTION
						,@BatchId ,@INITIAL_CALCULATED_AMOUNT - @CURRENT_RATE
						,AVB.[AvailableBalance] ,AVB.[AvailableBalance] + (@INITIAL_CALCULATED_AMOUNT - @CURRENT_RATE)
						,AVB.[CountableBalance] ,AVB.[CountableBalance] 
						,GETUTCDATE()
					FROM [dbo].[AccountVCCBalance] AVB (NOLOCK)
					WHERE [InstallationId] = @CURRENT_INSTALLATION_ID 
						AND [VCC] = @CURRENT_VCC
						AND [SystemService] = @CURRENT_SYSTEM_SERVICE

			-- ACTUALIZAR EL BATCH COMO ISFINISHED TRUE EN CASO NO HAYA SIDO MARCADO TODAVIA
			UPDATE [dbo].[AccountSubAccountVCCSummaryTransaction]
				SET [IsFinished] = 1,
					[EndDate] = CASE WHEN @ClosedDate IS NULL THEN [EndDate] ELSE @ClosedDate END,
					[Status] = CASE WHEN [Status] LIKE 'CANCELLED%' THEN 'FINISHED_BY_CANCELLED' ELSE 'FINISHED' END, 
					--[Status] = 'FINISHED', -- ESTE ESTADO SE CAMBIARA PARA DAR OPCION A FINISHED COMO COMPLETADO SIN INTERVENSION O FINISHED_BY_USER CUANDO EL USUARIO DECIDE TERMINARLO
					[LasModifiedDate] = GETUTCDATE(),
					[TimeStamp] = GETUTCDATE()
				WHERE [InstallationId] = @CURRENT_INSTALLATION_ID 
					AND [BatchID] = @BatchId
					AND [VCC] = @CURRENT_VCC
					AND [SystemService] = @CURRENT_SYSTEM_SERVICE
					--AND [IsFinished] = 0
					AND [Status] NOT IN ('FINISHED', 'FINISHED_BY_CANCELLED')

			FETCH NEXT FROM MY_CURSOR INTO @CURRENT_INSTALLATION_ID, @CURRENT_VCC, @CURRENT_SYSTEM_SERVICE, @CURRENT_RATE
		END
		CLOSE MY_CURSOR
		DEALLOCATE MY_CURSOR

		-- RETORNAMOS RESULTADO 1
		SELECT 1 'result'

		-- COMPLETAMOS LA TRANSACCION
		COMMIT TRANSACTION

    END TRY
    BEGIN CATCH
        IF @@TRANCOUNT > 0 ROLLBACK TRANSACTION

        DECLARE @Message nvarchar(2048) = ERROR_MESSAGE()
        DECLARE @Severity integer = ERROR_SEVERITY()
        DECLARE @State integer = ERROR_STATE()

        RAISERROR(@Message, @Severity, @State)
        RETURN -1
    END CATCH

	SET XACT_ABORT, NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO
