SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetAccountVccRates] (
	@InstallationId Varchar(100) = NULL, 
	@Vcc Varchar(50) = NULL,
	@ProcessId Varchar(50)  = NULL
) AS
BEGIN
	SET NOCOUNT ON

	-- DECLARAMOS UNA TABLE PARA ALMACENAR EL RESULTADO DEL SP
	DECLARE @SP_RESULT AS TABLE ([CAASPR] DECIMAL(5, 2), [CAARPM] DECIMAL(5, 2), [CABSR] DECIMAL(5, 2))
		
	-- EJECUTAMOS EL SP QUE RECUPERA EL NUMERO DE REQUEST PROMEDIO POR MINUTO Y GUARDAMOS EL RESULTADO EN LA TABLA CREADA
	INSERT @SP_RESULT EXEC [dbo].[GetCognitiveAnalysisAverageRequestPerMinute] @InstallationId, @Vcc 
	--EXEC [dbo].[GetCognitiveAnalysisAverageRequestPerMinute] @InstallationId, @Vcc 

	DECLARE @PROCESS_VALID_DURATION_TIME INT = 0
	DECLARE @PROCESS_ROUNDED_DURATION_TIME INT = 0

	DECLARE @PROCESS_CALL_PHRASES INT = 0
	DECLARE @PROCESS_TEXT_PHRASES INT = 0
	
	DECLARE @AVERAGE_COGNITIVE_ANALYSIS_SECONDS_PER_REQUEST DECIMAL(5, 2) = 0
	DECLARE @AVERAGE_COGNITIVE_ANALYSIS_REQUEST_PER_MINUTE DECIMAL(5, 2) = 0
	
	-- *TASA DE SEGURO DE FACTURACION
	DECLARE @COGNITIVE_ANALYSIS_CALL_BILLING_SECURE_RATE DECIMAL(5, 2) = 0
	DECLARE @COGNITIVE_ANALYSIS_TEXT_BILLING_SECURE_RATE DECIMAL(5, 2) = 0
	
	-- DECLARAMOS UN FLAG PARA INDICAR SI ES UN PROCESS PREVIO A LA IMPLEMENTACION DE CANALES
	DECLARE @IS_POST_CHANNEL BIT = 0

	-- EN CASO DE RECIBIR PROCESSID, BUSCAMOS EL BATCH RELACIONADO Y RECUPERAMOS SU VALIDDURATIONTIME
	IF @Processid IS NOT NULL
	BEGIN
		-- OBTENEMOS EL TOTAL DEL TIEMPO VALIDO Y DEL REDONDEADO EN MINUTOS DE CADA AUDIO
		SELECT @PROCESS_VALID_DURATION_TIME = (ASAVST.[ValidDurationTime] + ASAVST.[AlreadyTranscribedDurationTime]),
				@PROCESS_ROUNDED_DURATION_TIME = (ASAVST.[RoundedDurationTime] + ASAVST.[AlreadyTranscribedRoundedDurationTime]),
				@PROCESS_CALL_PHRASES = (ASAVST.[TotalCallPhrases] + ASAVST.[AlreadyCallPhrases]),
				@PROCESS_TEXT_PHRASES = (ASAVST.[TotalTextPhrases] + ASAVST.[AlreadyTextPhrases])
			FROM [dbo].[AccountSubAccountVCCSummaryTransaction] ASAVST (NOLOCK) 
			WHERE ASAVST.[InstallationId] = @InstallationId
				AND ASAVST.[VCC] = @Vcc
				AND ASAVST.[BatchTypeName] = 'TRANSCRIPTION'
				AND ASAVST.[BatchTypeID] = @ProcessId

		IF EXISTS (
			SELECT TOP 1 1 
				FROM [dbo].[AccountSubAccountVCCSummaryTransaction] ASAVST (NOLOCK), [dbo].[GeneralSecureRate] GSR (NOLOCK)
				WHERE ASAVST.[BatchTypeName] = 'TRANSCRIPTION'
					AND ASAVST.[CreatedDate] > GSR.[CreatedDate]
					AND ASAVST.[BatchTypeId] = @ProcessId
		)
			SET @IS_POST_CHANNEL = 1

		-- SEGUIMOS SEGUN EL PROCESS ES PREVIO O NO
		IF @IS_POST_CHANNEL = 1
		BEGIN
			-- ES UN PROCESS CREADO CUANDO SE USAN INTERACCIONES TIPO CALL Y TEXT
			SELECT TOP 1 @COGNITIVE_ANALYSIS_CALL_BILLING_SECURE_RATE = GSR.[SpeechCognitiveAnalysisSecureRate],
							@COGNITIVE_ANALYSIS_TEXT_BILLING_SECURE_RATE = GSR.[SpeechCognitiveAnalysisSecureRate]
				FROM [dbo].[GeneralSecureRate] GSR (NOLOCK)

			-- DESPUES DE LA IMPLEMENTACION DE CANALES REGISTRAMOS EN LAS TABLAS DE SPEECH Y BILLING LAS FRASES POR AUDIO EN LA TABLA DETAILED, ESTO DE DEBE LLEVAR A LA TABLA SUMMARY
		END
		ELSE
		BEGIN
			-- ES UN PROCESS CREADO CUANDO SOLO USABAN INTERACCIONES TIPO CALL, POR ESO SOLO NECESITA MARGEN PARA CALL 
			SELECT TOP 1 @COGNITIVE_ANALYSIS_CALL_BILLING_SECURE_RATE = GSR.[QualityCallCognitiveAnalysisSecureRate]
				FROM [dbo].[GeneralSecureRate] GSR (NOLOCK)
			
			-- ANTES DE LA IMPLEMENTACION DE CANALES NO SE REGISTRABAN LA CANTIDAD DE FRASES POR AUDIO EN TABLA DETAILED
		END
	END 
	ELSE 
	BEGIN
		-- PLAN DE EVALUACION USA LAS TASAS DE ANALISIS COGNITIVO
		SELECT TOP 1 @COGNITIVE_ANALYSIS_CALL_BILLING_SECURE_RATE = GSR.[QualityCallCognitiveAnalysisSecureRate],
						@COGNITIVE_ANALYSIS_TEXT_BILLING_SECURE_RATE = GSR.[QualityTextCognitiveAnalysisSecureRate]
			FROM [dbo].[GeneralSecureRate] GSR (NOLOCK)
	END

	-- RECUPERAMOS LOS DATOS DE REQUEST PROMEDIO Y TASA DE SEGURIDAD RECUPERADAS EN EL SP
	SELECT TOP 1 @AVERAGE_COGNITIVE_ANALYSIS_SECONDS_PER_REQUEST = SPR.[CAASPR],
			@AVERAGE_COGNITIVE_ANALYSIS_REQUEST_PER_MINUTE = SPR.[CAARPM]
			--@COGNITIVE_ANALYSIS_CALL_BILLING_SECURE_RATE = SPR.[CABSR] -- DEJARA DE USARSE PORQUE AHORA SE TOMA DE LA BD
		FROM @SP_RESULT SPR 

	-- HACEMOS LA CONSULTA A LA BASE DE DATOS
	SELECT AV.[TranscriptionRatePerHour]							'transcriptionRatePerHour',
		AV.[TranscriptionRatePerSecond]								'transcriptionRatePerSecond',
		AV.[SemanticAnalysisRatePerTransaction]						'semanticAnalysisRatePerTransaction',
		AV.[CognitiveAnalysisRatePerTransaction]					'cognitiveAnalysisRatePerTransaction',
		--(SELECT [CAARPM] FROM @SP_RESULT)							'cognitiveAnalysisAverageRequestPerMinute'
		@PROCESS_VALID_DURATION_TIME								'processTotalValidDurationTime',
		@PROCESS_ROUNDED_DURATION_TIME								'processRoundedValidDurationTime', 
		@AVERAGE_COGNITIVE_ANALYSIS_SECONDS_PER_REQUEST				'cognitiveAnalysisAverageSecondsPerRequest',
		@AVERAGE_COGNITIVE_ANALYSIS_REQUEST_PER_MINUTE				'cognitiveAnalysisAverageRequestPerMinute',
		@COGNITIVE_ANALYSIS_CALL_BILLING_SECURE_RATE				'cognitiveAnalysisBillingSecureRate',
		@COGNITIVE_ANALYSIS_TEXT_BILLING_SECURE_RATE				'cognitiveAnalysisTextBillingSecureRate',
		@PROCESS_CALL_PHRASES										'processCallPhrases',
		@PROCESS_TEXT_PHRASES										'processTextPhrases',
		@IS_POST_CHANNEL											'isProcessPostChannel'
	FROM dbo.[AccountVCC] AV (NOLOCK)
	WHERE AV.[InstallationId] = @InstallationId
		AND AV.[VCC] = @Vcc
	
	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO
