SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetCognitiveAnalysisAverageRequestPerMinute](
	@InstallationId VARCHAR(50) = NULL,
	@VCC VARCHAR(50) = NULL
)
AS
BEGIN
	-- ESTE SP TIENE LA FINALIDAD DE RETORNAR UN NUMERO PROMEDIO DE REQUEST DE ANALISIS COGNITIVO EN UN MINUTO
	-- USADO POR [dbo].[AccountSubAccountVCCBatchSummaryCreate] Y [dbo].[GetAccountVccRates]
	SET XACT_ABORT, NOCOUNT ON

	-- INICIAMOS UN TRY PARA EL CASO CAIGA EN CATCH HAGAMOS ROLLBACK
    BEGIN TRY
		-- DECLARAMOS UNA VARIABLE PARA ALMACENAR EL VALOR DE SEGUNDOS POR REQUEST
		DECLARE @SECONDS_PER_REQUEST DECIMAL(5,2) = 0

		-- DECLARAMOS UNA VARIABLE PARA ALMACENAR EL VALOR DE REQUEST POR MINUTO
		DECLARE @REQUEST_PER_MINUTE DECIMAL(5,2) = 0
		
		-- *TASA DE SEGURO DE FACTURACION(DECIMAL): SI SE CAMBIA ESTE VALOR, SE DEBERA CAMBIAR TAMBIEN EN EL SP [dbo].[AccountSubAccountVCCBatchSummaryCreate]
		DECLARE @COGNITIVE_ANALYSIS_BILLING_SECURE_RATE DECIMAL(5, 2) = 0.15 -- <-SE BORRARA

		-- DECLARAMOS LAS VARIABLES PARA LOS RESULTADOS TOTALES
		DECLARE @TOTAL_AUDIOS DECIMAL(18, 5) = 0
		DECLARE @TOTAL_DURATION_TIME DECIMAL(18, 5) = 0
		DECLARE @TOTAL_VALID_DURATION_TIME DECIMAL(18, 5) = 0
		DECLARE @TOTAL_ROUNDED_DURATION_TIME DECIMAL(18, 5) = 0
		DECLARE @TOTAL_VALID_REQUEST DECIMAL(18, 5) = 0
					
		-- RECUPERAMOS LOS VALORES TOTALES DE LOS ULTIMOS SIETE DIAS
		SELECT TOP 100 @TOTAL_AUDIOS = SUM(ASAVST.[TotalAudio]), 
				@TOTAL_DURATION_TIME = SUM(ASAVST.[DurationTime] + ASAVST.[AlreadyDurationTime]),
				@TOTAL_VALID_DURATION_TIME = SUM(CASE WHEN (ASAVST.[ValidDurationTime] + ASAVST.[AlreadyTranscribedDurationTime]) > 60 
												THEN (ASAVST.[ValidDurationTime] + ASAVST.[AlreadyTranscribedDurationTime])
												ELSE 60 END), 
				@TOTAL_ROUNDED_DURATION_TIME = SUM(CASE WHEN (ASAVST.[RoundedDurationTime] + ASAVST.[AlreadyTranscribedRoundedDurationTime]) > 60 
													THEN (ASAVST.[RoundedDurationTime] + ASAVST.[AlreadyTranscribedRoundedDurationTime]) 
													ELSE 60 END), 
				@TOTAL_VALID_REQUEST = SUM(CASE WHEN (ASAVDT.[TotalValidRequest] + ASAVDT.[TotalInvalidRequest]) < (ASAVST.[TotalAudio]) 
											THEN ASAVST.[TotalAudio] 
											ELSE (ASAVDT.[TotalValidRequest] + ASAVDT.[TotalInvalidRequest]) END)
			FROM [dbo].[AccountSubAccountVCCSummaryTransaction] ASAVST (NOLOCK) 
			INNER JOIN [dbo].[AccountSubAccountVCCDetailedTransaction] ASAVDT (NOLOCK) 
				ON ASAVDT.[InstallationId] = ASAVST.[InstallationId]
					AND ASAVDT.[BatchId] = ASAVST.[BatchID]
					AND ASAVDT.[SystemService] = ASAVST.[SystemService]
			WHERE ASAVST.[InstallationId] = @InstallationId
				AND ASAVST.[VCC] = @VCC 
				AND ASAVST.[SystemService] = 'COGNITIVE_ANALYSIS'
				AND ASAVDT.[Description] = 'SUCCED'
				AND ASAVST.[CreatedDate] >= '2020-05-01T00:00:00Z'

		/*
		SELECT @TOTAL_AUDIOS 'audios', 
				@TOTAL_VALID_REQUEST 'requests', 
				@TOTAL_DURATION_TIME 'durationTime',
				@TOTAL_VALID_DURATION_TIME 'validTime', 
				@TOTAL_ROUNDED_DURATION_TIME 'roundedTime', 
				(60.00 * @TOTAL_VALID_REQUEST) / @TOTAL_VALID_DURATION_TIME 'validTimeAverage',
				(60.00 * @TOTAL_VALID_REQUEST) / @TOTAL_ROUNDED_DURATION_TIME 'roundedTimeAverage'
		*/
			
		-- VALIDAMOS QUE LOS VALORES SEAN VALIDOS
		IF @TOTAL_AUDIOS IS NULL OR @TOTAL_AUDIOS = 0 
			OR @TOTAL_DURATION_TIME IS NULL OR @TOTAL_DURATION_TIME = 0 
			OR @TOTAL_VALID_DURATION_TIME IS NULL OR @TOTAL_VALID_DURATION_TIME = 0 
			OR @TOTAL_ROUNDED_DURATION_TIME IS NULL OR @TOTAL_ROUNDED_DURATION_TIME = 0
			OR @TOTAL_VALID_REQUEST  IS NULL OR @TOTAL_VALID_REQUEST = 0
		BEGIN
			-- TOMAMOS EL VALOR POR DEFECTO (O VALOR BASE) PARA SEGUNDOS POR REQUEST
			SET @SECONDS_PER_REQUEST = 30

			-- TOMAMOS EL VALOR POR DEFECTO (O VALOR BASE) PARA REQUEST POR MINUTO
			SET @REQUEST_PER_MINUTE = 2
		END
		ELSE
		BEGIN
			-- CALCULAMOSEL VALOR PROMEDIO DE SEGUNDOS POR REQUEST SEGUN EL HISTORICO
			SET @SECONDS_PER_REQUEST = @TOTAL_DURATION_TIME / @TOTAL_VALID_REQUEST

			-- CALCULAMOS EL VALOR PROMEDIO DE REQUEST POR MINUTO SEGUN EL HISTORICO
			--SET @REQUEST_PER_MINUTE = CEILING ((60.00 * @TOTAL_VALID_REQUEST) / @TOTAL_VALID_DURATION_TIME)
			SET @REQUEST_PER_MINUTE = (60.00 * @TOTAL_VALID_REQUEST) / @TOTAL_VALID_DURATION_TIME
			--SET @REQUEST_PER_MINUTE = CEILING ((60.00 * @TOTAL_VALID_REQUEST) / @TOTAL_ROUNDED_DURATION_TIME)
		END

		-- INSERTAMOS EL NUEVO PROMEDIO DE REQUEST EN LA TABLA HISTORICA
		INSERT INTO [dbo].[CognitiveAnalysisHistory] 
			([InstallationId], [VCC], [AverageRequestPerMinute])
		VALUES (@InstallationId, @VCC, @REQUEST_PER_MINUTE)
		
		-- RECUPERAMOS EL VALOR DE REQUEST PROMEDIO POR MINUTO
		SELECT @SECONDS_PER_REQUEST 'CAASPR', 
				@REQUEST_PER_MINUTE 'CAARPM', 
				@COGNITIVE_ANALYSIS_BILLING_SECURE_RATE 'CABSR'

	END TRY
    BEGIN CATCH
        -- IF @@TRANCOUNT > 0 ROLLBACK TRANSACTION

        DECLARE @Message nvarchar(2048) = ERROR_MESSAGE()
        DECLARE @Severity integer = ERROR_SEVERITY()
        DECLARE @State integer = ERROR_STATE()

        RAISERROR(@Message, @Severity, @State)
        RETURN -1
    END CATCH

	SET XACT_ABORT, NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO
