SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[ReportByAccountSubAccountCharge] (
	@PlatformHash VARCHAR(100)=NULL,
	@StartDate DATETIME = NULL,
	@EndDate DATETIME = NULL,
	@InstallationId VARCHAR(100)=NULL,
	@AccountId VARCHAR(50) = NULL,
	@SubAccountId VARCHAR(50) = NULL
	)
AS
BEGIN
	-- ESTE PROCEDURE TIENE POR FINALIDAD RETORNAR EL REPORTE DE RECARGA DE SALDOS
	SET NOCOUNT ON

	-- INICIAMOS UN TRY PARA EL CASO CAIGA EN CATCH HAGAMOS ROLLBACK
    BEGIN TRY
	
		-- EN EL CASO SUBACCOUNT SEA NULL
		IF @SubAccountId IS NULL
		BEGIN

			WITH CTE_CHARGE_REPORT AS (
				SELECT ASAVBM.[TimeStamp]											'COLUMN_DATE',
						A.[InstallationId]											'COLUMN_PLATFORM',
						A.[AccountId]												'COLUMN_ACCOUNT',
						A.[SubAccountId]											'COLUMN_SUBACCOUNT',
						ASAVBM.[CreatedByUserId]									'COLUMN_CREATED_BY',	
						ASAVBM.[PrevCountableBalance]								'COLUMN_PREVIOUS_COUNTABLE_BALANCE',
						ASAVBM.[Amount]												'COUMNN_AMOUNT',
						A.[TranscriptionRatePerHour]								'COLUMN_TRANSCRIPTION_RATE_PER_HOUR',
						-- A.[TranscriptionRatePerSecond]								'COLUMN_TRANSCRIPTION_RATE_PER_SECOND',
						A.[SemanticAnalysisRatePerTransaction]						'COLUMN_SEMANTIC_ANALYSIS_RATE_PER_TRANSACTION',
						A.[CognitiveAnalysisRatePerTransaction]						'COLUMN_COGNITIVE_ANALYSIS_RATE_PER_TRANSACTION',
						STUFF((SELECT ', ' + AV.[VCC]
								FROM [dbo].[AccountVCC] AV (NOLOCK)
								WHERE AV.[InstallationId] = A.[InstallationId]
									AND AV.[AccountId] = A.[AccountId]
									AND AV.[SubAccountId] IS NULL
								ORDER BY AV.[VCC]
								FOR XML PATH('')), 1, 1, '' ) AS					'COLUMN_RELATED_VCCS'
					FROM [dbo].[Account] A (NOLOCK)
					LEFT JOIN [dbo].[AccountSubAccountVCCBalanceMovement] ASAVBM (NOLOCK) 
						ON ASAVBM.[InstallationId] = A.[InstallationId]
							AND ASAVBM.[AccountId] = A.[AccountId]
							AND ASAVBM.[SubAccountId] IS NULL
							AND ASAVBM.[VCC]  IS NULL 
							AND ASAVBM.[Wallet] = 'FREE'
							AND ASAVBM.[Type] = 'CHARGE'
					WHERE A.[InstallationId] = @InstallationId
						AND A.[AccountId] = @AccountId
						AND A.[SubAccountId] IS NULL
						AND ASAVBM.[TimeStamp] BETWEEN @StartDate AND @EndDate 
				UNION
				SELECT ASAVBM.[TimeStamp]											'COLUMN_DATE',
						A.[InstallationId]											'COLUMN_PLATFORM',
						A.[AccountId]												'COLUMN_ACCOUNT',
						A.[SubAccountId]											'COLUMN_SUBACCOUNT',
						ASAVBM.[CreatedByUserId]									'COLUMN_CREATED_BY',	
						ASAVBM.[PrevCountableBalance]								'COLUMN_PREVIOUS_COUNTABLE_BALANCE',
						ASAVBM.[Amount]												'COUMNN_AMOUNT',
						A.[TranscriptionRatePerHour]								'COLUMN_TRANSCRIPTION_RATE_PER_HOUR',
						-- A.[TranscriptionRatePerSecond]								'COLUMN_TRANSCRIPTION_RATE_PER_SECOND',
						A.[SemanticAnalysisRatePerTransaction]						'COLUMN_SEMANTIC_ANALYSIS_RATE_PER_TRANSACTION',
						A.[CognitiveAnalysisRatePerTransaction]						'COLUMN_COGNITIVE_ANALYSIS_RATE_PER_TRANSACTION',
						STUFF((SELECT ', ' + AV.[VCC]
								FROM [dbo].[AccountVCC] AV (NOLOCK)
								WHERE AV.[InstallationId] = A.[InstallationId]
									AND AV.[AccountId] = A.[AccountId]
									AND AV.[SubAccountId] = A.[SubAccountId]
								ORDER BY AV.[VCC]
								FOR XML PATH('')), 1, 1, '' ) AS					'COLUMN_RELATED_VCCS'
					FROM [dbo].[Account] A (NOLOCK)
					LEFT JOIN [dbo].[AccountSubAccountVCCBalanceMovement] ASAVBM (NOLOCK) 
						ON ASAVBM.[InstallationId] = A.[InstallationId]
							AND ASAVBM.[AccountId] = A.[AccountId]
							AND ASAVBM.[SubAccountId] = A.[SubAccountId]
							AND ASAVBM.[VCC]  IS NULL 
							AND ASAVBM.[Wallet] = 'FREE'
							AND ASAVBM.[Type] = 'CHARGE'
					WHERE A.[InstallationId] = @InstallationId
						AND A.[AccountId] = @AccountId
						AND A.[SubAccountId] IS NOT NULL
						AND ASAVBM.[TimeStamp] BETWEEN @StartDate AND @EndDate 
			)

			-- EL CTE SOLO PUEDE SER INVOCADO UNA VEZ POR ESO LO PONEMOS EN EL SELECT FINAL DONDE EL CONTADOR(RN) ES 1
			SELECT * 
				FROM CTE_CHARGE_REPORT
				ORDER BY [COLUMN_DATE] DESC

		END
		ELSE
		-- EN EL CASO SUBACCOUNT LLEGUE CON UN VALOR
		BEGIN
			SELECT ASAVBM.[TimeStamp]											'COLUMN_DATE',
					A.[InstallationId]											'COLUMN_PLATFORM',
					A.[AccountId]												'COLUMN_ACCOUNT',
					A.[SubAccountId]											'COLUMN_SUBACCOUNT',
					ASAVBM.[CreatedByUserId]									'COLUMN_CREATED_BY',	
					ASAVBM.[PrevCountableBalance]								'COLUMN_PREVIOUS_COUNTABLE_BALANCE',
					ASAVBM.[Amount]												'COUMNN_AMOUNT',
					A.[TranscriptionRatePerHour]								'COLUMN_TRANSCRIPTION_RATE_PER_HOUR',
					-- A.[TranscriptionRatePerSecond]								'COLUMN_TRANSCRIPTION_RATE_PER_SECOND',
					A.[SemanticAnalysisRatePerTransaction]						'COLUMN_SEMANTIC_ANALYSIS_RATE_PER_TRANSACTION',
					A.[CognitiveAnalysisRatePerTransaction]						'COLUMN_COGNITIVE_ANALYSIS_RATE_PER_TRANSACTION',
					STUFF((SELECT ', ' + AV.[VCC]
							FROM [dbo].[AccountVCC] AV (NOLOCK)
							WHERE AV.[InstallationId] = A.[InstallationId]
								AND AV.[AccountId] = A.[AccountId]
								AND AV.[SubAccountId] = A.[SubAccountId]
							ORDER BY AV.[VCC]
							FOR XML PATH('')), 1, 1, '' ) AS					'COLUMN_RELATED_VCCS'
				FROM [dbo].[Account] A (NOLOCK)
				LEFT JOIN [dbo].[AccountSubAccountVCCBalanceMovement] ASAVBM (NOLOCK) 
					ON ASAVBM.[InstallationId] = A.[InstallationId]
						AND ASAVBM.[AccountId] = A.[AccountId]
						AND ASAVBM.[SubAccountId]  = A.[SubAccountId]
						AND ASAVBM.[VCC]  IS NULL 
						AND ASAVBM.[Wallet] = 'FREE'
						AND ASAVBM.[Type] = 'CHARGE'
				WHERE A.[InstallationId] = @InstallationId
					AND A.[AccountId] = @AccountId
					AND A.[SubAccountId] = @SubAccountId 
					AND ASAVBM.[TimeStamp] BETWEEN @StartDate AND @EndDate 
				ORDER BY ASAVBM.[TimeStamp] DESC

		END 

	END TRY
    BEGIN CATCH
        DECLARE @Message nvarchar(2048) = ERROR_MESSAGE()
        DECLARE @Severity integer = ERROR_SEVERITY()
        DECLARE @State integer = ERROR_STATE()

        RAISERROR(@Message, @Severity, @State)
        RETURN -1

    END CATCH

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO
