SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[SaveAccountVCCProduct] (
	@PlatformHash Varchar(100) = NULL,
	@VCC Varchar(50) = NULL,
	@Product Varchar(250) = NULL,
	@LastModifiedByUserId Varchar(50) = NULL
	)
AS
BEGIN
	-- ESTE SP TIENE POR ACTUALIZAR LOS PRODUCTOS QUE POSEE UN VCC
	SET XACT_ABORT, NOCOUNT ON

	-- INICIAMOS UN TRY PARA EL CASO CAIGA EN CATCH HAGAMOS ROLLBACK
    BEGIN TRY
        -- INICIAMOS LA TRANSACCION
		BEGIN TRANSACTION

		-- VERIFICAMOS QUE TENEMOS LOS PARAMETROS NECESARIOS
		IF @PlatformHash IS NULL OR @VCC IS NULL OR @Product IS NULL
			-- LANZAMOS MENSAJE AVISANDO QUE FALTA ACCOUNTID
			RAISERROR('DATABABASE_ERROR_MISSING_PARAMETERS', 16, 16)

		-- DECLARAMOS VARIABLES PARA LLEVAR ACABO EL PROCESO
		DECLARE @INSTALLATION_ID VARCHAR(50) = NULL
		DECLARE @ACCOUNT_ID VARCHAR(50) = NULL
		DECLARE @SUBACCOUNT_ID VARCHAR(50) = NULL
		DECLARE @CURRENT_VCC_PRODUCT_ID VARCHAR(50) = NULL
		DECLARE @CURRENT_VCC_PRODUCT_NAME VARCHAR(50) = NULL
		DECLARE @CURRENT_VCC_PRODUCT_STATUS VARCHAR(50) = NULL

		-----------------------------------------------
		-- INICIAMOS LAS VALIDACIONES DE PLATAFORMA ---
		-----------------------------------------------

		-- VALIDAMOS SI EXISTE LA PLATAFORMA 
		IF NOT EXISTS (
			SELECT P.[IsActivated] 
				FROM [dbo].[Platform] P (NOLOCK) 
				WHERE P.[PlatformHash] = @PlatformHash
		)
			-- LANZAMOS MENSAJE AVISANDO QUE NO EXISTE LA PLATAFORMA
			RAISERROR('DATABABASE_ERROR_PLATFORM_DOES_NOT_EXIST', 16, 16)

		-- VALIDAMOS SI LA PLATAFORMA ESTA ACTIVADA
		IF NOT EXISTS (
			SELECT P.[IsActivated] 
				FROM [dbo].[Platform] P (NOLOCK) 
				WHERE P.[PlatformHash] = @PlatformHash
					AND P.[IsActivated] = 1
		)
			-- LANZAMOS MENSAJE AVISANDO QUE FALTA ACCOUNTID
			RAISERROR('DATABABASE_ERROR_PLATFORM_IS_NOT_ACTIVATED', 16, 16)

		-----------------------------------------------
		-- TERMINAMOS LAS VALIDACIONES DE PLATAFORMA --
		-----------------------------------------------
		
		-- ASIGNAMOS EL VALOR DE LA PLATAFORMA A UNA VARIABLE LOCAL
		SELECT @INSTALLATION_ID = P.[InstallationId] 
			FROM [dbo].[Platform] P (NOLOCK) 
			WHERE P.[PlatformHash] = @PlatformHash
				AND P.[IsActivated] = 1

		-----------------------------------------------
		-- INICIAMOS LAS VALIDACIONES DEL VCC ---------
		-----------------------------------------------

		-- VALIDAMOS SI EXISTE EL VCC PROPORCIONADO
		IF NOT EXISTS (
			SELECT AV.[VCC]
				FROM [dbo].[AccountVCC] AV (NOLOCK) 
				WHERE AV.[InstallationId] = @INSTALLATION_ID
					AND AV.[VCC] = @VCC
		)
			-- LANZAMOS MENSAJE AVISANDO QUE NO EXISTE LA PLATAFORMA
			RAISERROR('DATABABASE_ERROR_VCC_DOES_NOT_EXIST', 16, 16)

		-----------------------------------------------
		-- TERMINAMOS LAS VALIDACIONES DEL VCC ---------
		-----------------------------------------------

		-- ASIGNAMOS LOS VALORES DE CUENTA Y SUBCUENTA A LAS VARIABLES LOCALES
		SELECT @ACCOUNT_ID = AV.[AccountId],
				@SUBACCOUNT_ID = AV.[SubAccountId]
			FROM [dbo].[AccountVCC] AV (NOLOCK) 
			WHERE AV.[InstallationId] = @INSTALLATION_ID
				AND AV.[VCC] = @VCC

		-- DECLARAMOS UNA VARIABLE PARA ALMACENAR TODOS LOS PRODUCTOS ASOCIADOS ACTUALMENTE AL VCC
		DECLARE MY_CURSOR CURSOR 
		LOCAL STATIC READ_ONLY FORWARD_ONLY
			FOR 
			SELECT PL.[Product],
					ISNULL(PLU.[Enabled], NULL),
					ISNULL(PLU.[ID], NULL)
				FROM [dbo].[PlatformLicense] PL (NOLOCK)
				LEFT JOIN [dbo].[PlatformLicenseUsage] PLU (NOLOCK) 
					ON PLU.[InstallationId] = PL.[InstallationId]
						AND PLU.[Product] = PL.[Product]
						AND PLU.[Enabled] = 1
						AND PLU.[VCC] = @VCC
				WHERE PL.[PlatformHash] = @PlatformHash
					AND PL.[InstallationId] = @INSTALLATION_ID

		-- ABRIMOS Y RECORREMOS EL ARRAY DE PRODUCTOS
		OPEN MY_CURSOR
		FETCH NEXT FROM MY_CURSOR INTO @CURRENT_VCC_PRODUCT_NAME, @CURRENT_VCC_PRODUCT_STATUS, @CURRENT_VCC_PRODUCT_ID
		WHILE @@FETCH_STATUS = 0
		BEGIN 
			
			-- VALIDAMOS SI EL PRODUCTO RECORRDIDO EXISTE EN LA LISTA RECIBIDA
			IF @CURRENT_VCC_PRODUCT_NAME IN (SELECT VALUE FROM [dbo].[split](@Product, ','))
			BEGIN
				-- EL PRODUCTO RECORRIDO EXISTE EN LA LISTA DE PRODUCTOS ACTIVOS RECIBIDA

				-- SI EL STATUS DEL PRODUCTO RECORRIDO ES 0, DEBEMOS ACTUALIZARLO A 1
				IF @CURRENT_VCC_PRODUCT_STATUS = 0 
				BEGIN
					-- PRINT 'ACTUALIZAR A 1': ACTUALIZAMOS EL PRODUCTO POR SU ID UNICO
					EXEC AddAccountVCCProduct @InstallationId = @INSTALLATION_ID, @AccountId = @ACCOUNT_ID, @SubAccountId = @SUBACCOUNT_ID, @VCC = @VCC, @Product = @CURRENT_VCC_PRODUCT_NAME

				END
				-- SI EL STATUS DEL PRODUCTO RECORRIDO ES NULL, DEBEMOS ACTUALIZARLO A 0
				ELSE IF @CURRENT_VCC_PRODUCT_STATUS IS NULL
				BEGIN
					-- PRINT 'AGREGAR PRODUCTO': EJECUTAMOS EL PROCEDURE QUE INSERTA VCCPRODUCT
					EXEC AddAccountVCCProduct @InstallationId = @INSTALLATION_ID, @AccountId = @ACCOUNT_ID, @SubAccountId = @SUBACCOUNT_ID, @VCC = @VCC, @Product = @CURRENT_VCC_PRODUCT_NAME
				END
			END
			ELSE 
			BEGIN
				-- EL PRODUCTO RECORRIDO NO EXISTE EN LA LISTA DE PRODUCTOS ACTIVOS RECIBIDA

				-- SI EL STATUS DEL PRODUCTO RECORRIDO ES 1, DEBEMOS ACTUALIZARLO A 0
				IF @CURRENT_VCC_PRODUCT_STATUS = 1 
				BEGIN
					-- PRINT 'ACTUALIZAR A 0': ACTUALIZAMOS EL PRODUCTO POR SU ID UNICO
					UPDATE [dbo].[PlatformLicenseUsage] 
						SET [Enabled] = 0,
							[LastModifiedDate] = GETUTCDATE(),
							[LastModifiedByUserId] = @LastModifiedByUserId
						WHERE [ID] = @CURRENT_VCC_PRODUCT_ID
					-- LANZAMOS MENSAJE DE ERROR SI EL UPDATE FALLO
					IF @@ROWCOUNT = 0
						RAISERROR('DATABABASE_ERROR_PLATFORM_LICENSE_USAGE_UPDATE_FAILED', 16, 16)

					-- AL LIBERAR UNA LICENCIA, DEBEMOS SUMARLA A LA TABLA PLATFORMLICENSE
					
					-- ACTUALIZAMOS EL CAMPO [AssignedLicense] DE LA TABLA dbo.[PlatformLicense] Y DESBLOQUEAMOS EL REGISTRO
					UPDATE dbo.[PlatformLicense] 
						SET [AssignedLicense] = [AssignedLicense] - 1,
							[LastModifiedByUserId] = @LastModifiedByUserId,
							[LastModifiedDate] = GETUTCDATE(),
							[TimeStamp] = GETUTCDATE()
						WHERE [InstallationId] = @INSTALLATION_ID
							AND [Product] = @CURRENT_VCC_PRODUCT_NAME
					-- LANZAMOS MENSAJE DE ERROR SI EL UPDATE FALLO
					IF @@ROWCOUNT = 0
						RAISERROR('DATABABASE_ERROR_PLATFORM_LICENSE_UPDATE_FAILED', 16, 16)
				END
			END
			
			--Do something with Id here
			FETCH NEXT FROM MY_CURSOR INTO @CURRENT_VCC_PRODUCT_NAME, @CURRENT_VCC_PRODUCT_STATUS, @CURRENT_VCC_PRODUCT_ID
		END
		CLOSE MY_CURSOR
		DEALLOCATE MY_CURSOR

		SELECT 1 'result'

        COMMIT TRANSACTION;
    END TRY
    BEGIN CATCH
        IF @@TRANCOUNT > 0 ROLLBACK TRANSACTION

        DECLARE @Message nvarchar(2048) = ERROR_MESSAGE()
        DECLARE @Severity integer = ERROR_SEVERITY()
        DECLARE @State integer = ERROR_STATE()

        RAISERROR(@Message, @Severity, @State)
        RETURN -1
    END CATCH

	SET XACT_ABORT, NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO
