SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[SavePlatformData] (
	@PlatformHash VARCHAR(100) = NULL,
	@ClientName VARCHAR(50) = NULL,
	@ClientContactName VARCHAR(50) = NULL,
	@ClientContactEmail VARCHAR(50) = NULL,
	@TranscriptionRatePerHour DECIMAL(18,5) = 0,
	@TranscriptionRatePerSecond DECIMAL(18,5) = 0,
	@SemanticAnalysisRatePerTransaction DECIMAL(18,5) = 0,
	@CognitiveAnalysisRatePerTransaction DECIMAL(18,5) = 0,
	@User VARCHAR(50) = NULL
	)
AS
BEGIN
	SET XACT_ABORT, NOCOUNT ON

	-- ABRIMOS EL TRY PARA LA TRANSACCION
    BEGIN TRY
		BEGIN TRANSACTION
			-- VALIDAMOS QUE EL COSTO DE TRANSCRIPCION POR HORA SEA MINIMO DE 0.036
			IF @TranscriptionRatePerHour < 0.036
				RAISERROR('DATABABASE_ERROR_TRANSCRIPTION_RATE_PER_HOUR_MINIMUN_EXCEPTION', 16, 16)

			-- ACTUALIZAMOS LOS CAMPOS CLIENTCONTACTEMAIL, TRANSCRIPTIONRATEPERHOUR, SEMANTICANALYSISRATEPERTRANSACTION, COGNITIVEANALYSISRATEPERTRANSACTION, LASTMODIFIEDDATE, LASTMODIFIEDBYUSERID y TIMESTAMP DE ACUERDO EL PLATFORMHASH.
			UPDATE [dbo].[PlatformData] SET
				[ClientContactEmail] = @ClientContactEmail,
				[TranscriptionRatePerHour] = @TranscriptionRatePerHour,
				[SemanticAnalysisRatePerTransaction] = @SemanticAnalysisRatePerTransaction,
				[CognitiveAnalysisRatePerTransaction] = @CognitiveAnalysisRatePerTransaction,
				[LastModifiedDate] = GETUTCDATE(),
				[TimeStamp] = GETUTCDATE()
			WHERE PlatformHash = @PlatformHash

			IF(@@ROWCOUNT = 0)
			BEGIN
				-- INSERTAMOS EL NUEVO REGISTRO EN LA TABLA PLATFORMDATA.
				INSERT INTO [dbo].[PlatformData]
					   ([PlatformHash]
					   ,[ClientName]
					   ,[ClientContactName]
					   ,[ClientContactEmail]
					   ,[TranscriptionRatePerHour]
					   ,[TranscriptionRatePerSecond]
					   ,[SemanticAnalysisRatePerTransaction]
					   ,[CognitiveAnalysisRatePerTransaction]
					   ,[CreatedDate]
					   ,[CreatedByUserId]
					   ,[TimeStamp])
				 VALUES
					   (@PlatformHash
					   ,@ClientName
					   ,@ClientContactName
					   ,@ClientContactEmail
					   ,@TranscriptionRatePerHour
					   ,@TranscriptionRatePerSecond
					   ,@SemanticAnalysisRatePerTransaction
					   ,@CognitiveAnalysisRatePerTransaction
					   ,GETUTCDATE()
					   ,@User
					   ,GETUTCDATE())

			END
	
			-- RETORNAMOS UN VALOR DE RESPUESTA
			SELECT 1 AS 'result'

		-- CONSOLIDAMOS LA TRANSACCION
		COMMIT TRANSACTION

	 -- TERMINAMOS EL TRY PARA LA TRANSACCION
    END TRY
	BEGIN CATCH
        IF @@TRANCOUNT > 0 ROLLBACK TRANSACTION

        DECLARE @Message nvarchar(2048) = ERROR_MESSAGE()
        DECLARE @Severity integer = ERROR_SEVERITY()
        DECLARE @State integer = ERROR_STATE()

        RAISERROR(@Message, @Severity, @State)
        RETURN -1
    END CATCH

    SET XACT_ABORT, NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO
