SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[ValidateApiUser] (
	@InstallationId VARCHAR(50),
	@VCC VARCHAR(50),
	@ApiUser VARCHAR(50),
	@Pin VARCHAR(250),
	@User VARCHAR(100)
)
AS
BEGIN
	SET NOCOUNT ON

	-- VALIDAMOS SI EXISTE EL USUARIO PROPORCIONADO
	IF NOT EXISTS (
		SELECT [InstallationId] 
		FROM dbo.[UserApi] UA 
		WHERE UA.[InstallationId] = @InstallationId 
			AND UA.[VCC] = @VCC 
			AND UA.[UserApi] = @ApiUser)
	BEGIN
		-- RETORNAMOS MENSAJE DE ERROR
		RAISERROR('DATABABASE_ERROR_NOT_FOUND_USER', 16, 16)
		RETURN
	END 

	-- VALIDAMOS SI LA CUENTA DEL USUARIO ESTA BLOQUEADA POR LIMITE DE INTENTOS
	IF EXISTS (
		SELECT [InstallationId] 
		FROM dbo.[UserApi] UA 
		WHERE UA.[InstallationId] = @InstallationId 
			AND UA.[VCC] = @VCC 
			AND UA.[UserApi] = @ApiUser
			AND UA.[IsLocked] = 1)
	BEGIN
		-- RETORNAMOS MENSAJE DE ERROR
		RAISERROR('DATABABASE_ERROR_FAILED_RETRY_LOCKED', 16, 16)
		RETURN
	END 
	
	-- VALIDAMOS SI LA CUENTA COINCIDE CON LAS CREDENCIALES PROPORCIONADAS
	IF NOT EXISTS (
		SELECT [InstallationId] 
		FROM dbo.[UserApi] UA 
		WHERE UA.[InstallationId] = @InstallationId 
			AND UA.[VCC] = @VCC 
			AND UA.[UserApi] = @ApiUser
			AND UA.[Pin] = @Pin
	)
	BEGIN
		-- SI NO HUBO COINCIDENCIA CON LAS CREDENCIALES, AUMENTAMOS EN 1 EL NUMERO DE INTENTOS FALLIDOS
		UPDATE dbo.[UserApi] 
			SET [FailedTry] = [FailedTry] + 1,
				[IsLocked] = CASE WHEN [FailedTry] + 1 >= 3 THEN 1 ELSE 0 END,
				[LastModifiedDate] = GETUTCDATE(),
				[LastModifiedByUserId] = @User,
				[LastModifiedByRemoteUser] = 1,
				[TimeStamp] = GETUTCDATE()
			WHERE [InstallationId] = @InstallationId 
				AND [VCC] = @VCC 
				AND [UserApi] = @ApiUser

		-- RETORNAMOS MENSAJE DE ERROR
		RAISERROR('DATABABASE_ERROR_CREDENTIALS_FAILED', 16, 16)
		RETURN
	END 
	ELSE 	
	BEGIN
		-- REINICIAMOS EL CONTADOR DE INTENTOS FALLIDOS
		UPDATE dbo.[UserApi] 
			SET [FailedTry] = 0,
				[LastModifiedDate] = GETUTCDATE(),
				[LastModifiedByUserId] = @User,
				[LastModifiedByRemoteUser] = 1,
				[TimeStamp] = GETUTCDATE()
			WHERE [InstallationId] = @InstallationId 
				AND [VCC] = @VCC 
				AND [UserApi] = @ApiUser

		-- SI ENCONTRO COINCIDENCIA CON LAS CREDENCIALES, RETORNAMOS UNA RESPUESTA GENERICA
		SELECT 1 'result'
	END 

	SET NOCOUNT OFF
END 
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO
